@@ -17,7 +17,6 @@
 
  
 
-/* $Id: render_handler_agg.cpp,v 1.127.2.2 2008/03/04 11:03:59 strk Exp $ */
 
 // Original version by Udo Giacomozzi and Hannes Mayr, 
 // INDUNET GmbH (www.indunet.it)
@@ -43,6 +42,10 @@ Status
     patterns          don't exist (they're converted at compile time by Flash!)
     widths            COMPLETE
     colors, alpha     COMPLETE
+    cap styles        DONE, but end cap style is ignored
+    join styles       COMPLETE
+    no-close flag     COMPLETE        
+  
     
   fills:
     solid fills       COMPLETE
@@ -172,6 +175,8 @@ AGG ressources
 #include "render_handler_agg_bitmap.h"
 #include "render_handler_agg_style.h"
 
+//#include <boost/foreach.hpp>
+
 #ifndef round
 #define round(x) rint(x)
 #endif
@@ -620,8 +625,8 @@ public:
     // by default allow drawing everywhere
     set_invalidated_region_world();
     
-    log_debug("initialized AGG buffer <%p>, %d bytes, %dx%d, rowsize is %d bytes", 
-      mem, size, x, y, rowstride);
+    log_debug(_("Initialized AGG buffer <%p>, %d bytes, %dx%d, rowsize is %d bytes"), 
+      (void*)mem, size, x, y, rowstride);
   }
   
 
@@ -690,10 +695,10 @@ public:
   {
   
     if (m_drawing_mask) 
-      log_debug("warning: rendering ended while drawing a mask");
+      log_debug(_("Warning: rendering ended while drawing a mask"));
       
     while (! m_alpha_mask.empty()) {
-      log_debug("warning: rendering ended while masks were still active");
+      log_debug(_("Warning: rendering ended while masks were still active"));
       disable_mask();      
     }
   
@@ -715,7 +720,7 @@ public:
 
   
   void  draw_line_strip(const void* coords, int vertex_count, const rgba& color,
-	                const matrix& line_mat)
+                  const matrix& line_mat)
   // Draw the line strip formed by the sequence of points.
   {
   
@@ -837,7 +842,7 @@ public:
   void draw_glyph(shape_character_def *def,
       const matrix& mat, const rgba& color, float /*pixel_scale*/) {
     
-    std::vector<path> paths;    
+    std::vector< Path<float> > paths;    
     apply_matrix_to_path(def->get_paths(), paths, mat);
     
     // convert to AGG paths
@@ -890,8 +895,8 @@ public:
     rect ch_bounds = def->get_bound();
 
     if (ch_bounds.is_null()) {
-      log_debug("warning: select_clipbounds encountered a character definition "
-        "with null bounds");
+      log_debug(_("Warning: select_clipbounds encountered a character definition "
+        "with null bounds"));
       return;
     }   
 
@@ -949,10 +954,11 @@ public:
     
     analyze_paths(def->get_paths(), have_shape, have_outline);
 
-    if (!have_shape && !have_outline) 
+    if (!have_shape && !have_outline)
       return; // invisible character
 
-    std::vector< path > paths;
+
+    std::vector< Path<float> > paths;
     std::vector< agg::path_storage > agg_paths;
     std::vector< agg::path_storage > agg_paths_rounded;
     
@@ -960,9 +966,9 @@ public:
     
     // Flash only aligns outlines. Probably this is done at rendering
     // level.
-    
+
     if (have_outline)
-      build_agg_paths_rounded(agg_paths_rounded, paths);
+      build_agg_paths_rounded(agg_paths_rounded, paths, line_styles);
     
     if (have_shape)
       build_agg_paths(agg_paths, paths);
@@ -979,7 +985,7 @@ public:
       select_clipbounds(def, mat);
       
       if (_clipbounds_selected.empty()) {
-        log_debug("warning: AGG renderer skipping a whole character");
+        log_debug(_("Warning: AGG renderer skipping a whole character"));
         return; // nothing to draw!?
       }
     
@@ -1046,53 +1052,31 @@ public:
     
   }
 
-
   /// Takes a path and translates it using the given matrix. The new path
   /// is stored in paths_out.  
   void apply_matrix_to_path(const std::vector<path> &paths_in, 
-    std::vector<path> &paths_out, const matrix &source_mat) {
+    std::vector< Path<float> > &paths_out, const matrix &source_mat) {
     
-    int pcount, ecount;
-    int pno, eno;
-
     matrix mat = stage_matrix;
     mat.concatenate(source_mat);
     
-    // copy path
-    paths_out = paths_in;    
-    pcount = paths_out.size();
-        
-    
-    for (pno=0; pno<pcount; pno++) {
-    
-      path &the_path = paths_out[pno];     
-      point oldpnt(the_path.ap.x, the_path.ap.y);
-      point newpnt;
-      mat.transform(&newpnt, oldpnt);
-      the_path.ap.x = newpnt.x;    
-      the_path.ap.y = newpnt.y;
-      
-      ecount = the_path.m_edges.size();
-      for (eno=0; eno<ecount; eno++) {
-      
-        edge &the_edge = the_path.m_edges[eno];
-        
-        oldpnt.x = the_edge.ap.x;
-        oldpnt.y = the_edge.ap.y;
-        mat.transform(&newpnt, oldpnt);
-        the_edge.ap.x = newpnt.x;
-        the_edge.ap.y = newpnt.y;
-        
-        oldpnt.x = the_edge.cp.x;
-        oldpnt.y = the_edge.cp.y;
-        mat.transform(&newpnt, oldpnt);
-        the_edge.cp.x = newpnt.x;
-        the_edge.cp.y = newpnt.y;
-      
-      }          
-      
-    } 
+    paths_out.reserve(paths_in.size());
     
+    typedef std::vector<path> PathVect;
+    for (PathVect::const_iterator i=paths_in.begin(), e=paths_in.end(); i!=e; ++i)
+    {
+  Path<float> floating_path = *i;
+  floating_path.transform(mat);
+  paths_out.push_back(floating_path);
+    }
+#if 0
+    BOOST_FOREACH(const path& in_path, paths_in) {
+      Path<float> floating_path = in_path;
+      floating_path.transform(mat);
+      paths_out.push_back(floating_path);
+    }
+#endif
+
   } // apply_matrix
 
 
@@ -1101,14 +1085,14 @@ public:
   /// layers of the same frame count. Flash combines them to one single shape.
   /// The problem with sub-shapes is, that outlines can be hidden by other
   /// layers so they must be rendered separately. 
-  unsigned int count_sub_shapes(const std::vector<path> &paths)
+  unsigned int count_sub_shapes(const std::vector< Path<float> > &paths)
   {
     unsigned int sscount=1;
     
     size_t pcount = paths.size();
     
     for (size_t pno=0; pno<pcount; pno++) {
-      const path &this_path = paths[pno];
+      const Path<float>& this_path = paths[pno];
       
       if (this_path.m_new_shape)
         sscount++;
@@ -1121,7 +1105,7 @@ public:
   /// Transposes Gnash paths to AGG paths, which can be used for both outlines
   /// and shapes. Subshapes are ignored (ie. all paths are converted). Converts 
   /// TWIPS to pixels on the fly.
-  void build_agg_paths(std::vector<agg::path_storage>& dest, const std::vector<path>& paths) {
+  void build_agg_paths(std::vector<agg::path_storage>& dest, const std::vector< Path<float> >& paths) {
   
     // Shift all coordinates a half pixel for correct results (the middle of
     // a pixel is at .5 / .5, ie. it's subpixel center) 
@@ -1133,7 +1117,7 @@ public:
     
     for (size_t pno=0; pno<pcount; pno++) {
       
-      const gnash::path& this_path = paths[pno];
+      const Path<float>& this_path = paths[pno];
       agg::path_storage& new_path = dest[pno];
       
       new_path.move_to(this_path.ap.x + subpixel_offset, 
@@ -1143,7 +1127,7 @@ public:
       
       for (size_t eno=0; eno<ecount; eno++) {
         
-        const edge& this_edge = this_path.m_edges[eno];
+        const Edge<float>& this_edge = this_path.m_edges[eno];
         
         if (this_edge.is_straight())
           new_path.line_to(this_edge.ap.x + subpixel_offset, 
@@ -1159,7 +1143,8 @@ public:
   } //build_agg_paths
   
   
-  // Version of build_agg_paths that uses rounded coordinates.
+  // Version of build_agg_paths that uses rounded coordinates (pixel hinting)
+  // for line styles that want it.  
   // This is used for outlines which are aligned to the pixel grid to avoid
   // anti-aliasing problems (a perfect horizontal line being drawn over two
   // lines and looking blurry). The proprietary player does this too.  
@@ -1170,11 +1155,15 @@ public:
   // Also, single segments of a path may be aligned or not depending on 
   // the segment properties (this matches MM player behaviour)
   //
+  // This function - in contrast to build_agg_paths() - also checks noClose 
+  // flag and automatically closes polygons.
+  //
   // TODO: Flash never aligns lines that are wider than 1 pixel on *screen*,
   // but we currently don't check the width.  
   void build_agg_paths_rounded(std::vector<agg::path_storage>& dest, 
-    const std::vector<path>& paths) {
-  
+    const std::vector< Path<float> >& paths, 
+    const std::vector<line_style>& line_styles) {
+
     // Shift all coordinates a half pixel for correct results (the middle of
     // a pixel is at .5 / .5, ie. it's subpixel center) 
     const float subpixel_offset = 0.5f;
@@ -1185,28 +1174,47 @@ public:
     
     for (size_t pno=0; pno<pcount; pno++) {
       
-      const gnash::path& this_path = paths[pno];
+      const Path<float>& this_path = paths[pno];
       agg::path_storage& new_path = dest[pno];
       
+      bool hinting=false, closed=false, hairline=false;
+      
+      if (this_path.m_line) {
+        const line_style& lstyle = line_styles[this_path.m_line-1];
+        
+        hinting = lstyle.doPixelHinting();
+        closed = this_path.isClosed() && !lstyle.noClose();
+        
+        // check if this line is a hairline ON SCREEN
+        // TODO: we currently only check for hairlines per definiton, not
+        // for thin lines that become hair lines due to scaling
+        if (lstyle.getThickness()<=20)
+          hairline = true;
+      }
+      
       float prev_ax = this_path.ap.x;
       float prev_ay = this_path.ap.y;  
       bool prev_align_x = true;
       bool prev_align_y = true;
       
       size_t ecount = this_path.m_edges.size();
+
+      // avoid extra edge when doing implicit close later
+      if (closed && ecount && 
+        this_path.m_edges.back().is_straight()) ecount--;      
       
       for (size_t eno=0; eno<ecount; eno++) {
         
-        const edge& this_edge = this_path.m_edges[eno];
+        const Edge<float>& this_edge = this_path.m_edges[eno];
         
         float this_ax = this_edge.ap.x;  
         float this_ay = this_edge.ap.y;  
         
-        if (this_edge.is_straight()) {
+        if (hinting || this_edge.is_straight()) {
         
           // candidate for alignment?
-          bool align_x = prev_ax == this_ax;
-          bool align_y = prev_ay == this_ay;
+          bool align_x = hinting || (hairline && (prev_ax == this_ax));
+          bool align_y = hinting || (hairline && (prev_ay == this_ay));
           
           if (align_x) 
             this_ax = round(this_ax);
@@ -1283,58 +1291,14 @@ public:
         prev_ay = this_ay;    
         
       } //for
+      
+      if (closed)
+        new_path.close_polygon();
     
     }
-        
   } //build_agg_paths_rounded
   
-  
-  // Builds vector strokes for paths
-  // WARNING 1 : This is not used and will probably be removed soon.
-  // WARNING 2 : Strokes vector returns pointers which are never freed.
-  void build_agg_strokes(std::vector<stroke_type*>& dest, 
-    std::vector<agg::path_storage>& agg_paths,
-    const std::vector<path> &paths,
-    const std::vector<line_style> &line_styles,
-    const matrix& linestyle_matrix) {
     
-    abort(); // should not be used currently
-    
-    size_t pcount=paths.size(); 
-    dest.resize(pcount);
-    
-    // use avg between x and y scale
-    const float stroke_scale = 
-      (fabsf(linestyle_matrix.get_x_scale()) + 
-       fabsf(linestyle_matrix.get_y_scale()))
-       / 2.0f
-      * get_stroke_scale();   
-    
-    for (size_t pno=0; pno<pcount; pno++) {
-          
-      agg::conv_curve<agg::path_storage> curve(agg_paths[pno]);
-      stroke_type* this_stroke = new stroke_type(curve);
-      
-      const gnash::path &this_path_gnash = paths[pno];
-       
-      const line_style& lstyle = line_styles[this_path_gnash.m_line-1];
-          
-      int width = lstyle.get_width();
-      if (width==1)
-        this_stroke->width(1);
-      else
-        this_stroke->width(width*stroke_scale);
-       
-      this_stroke->attach(curve);
-      this_stroke->line_cap(agg::round_cap);
-      this_stroke->line_join(agg::round_join);
-      
-      dest[pno] = this_stroke;
-    
-    }
-    
-  }
-  
   // Initializes the internal styles class for AGG renderer
   void build_agg_styles(agg_style_handler& sh, 
     const std::vector<fill_style>& fill_styles,
@@ -1439,7 +1403,7 @@ public:
   /// @param subshape_id
   ///    Defines which subshape to draw. -1 means all subshapes.
   ///
-  void draw_shape(int subshape_id, const std::vector<path> &paths,
+  void draw_shape(int subshape_id, const std::vector< Path<float> > &paths,
     const std::vector<agg::path_storage>& agg_paths,  
     agg_style_handler& sh, int even_odd) {
     
@@ -1473,7 +1437,7 @@ public:
   /// one with and one without an alpha mask. This makes drawing without masks
   /// much faster.  
   template <class scanline_type>
-  void draw_shape_impl(int subshape_id, const std::vector<path> &paths,
+  void draw_shape_impl(int subshape_id, const std::vector< Path<float> > &paths,
     const std::vector<agg::path_storage>& agg_paths,
     agg_style_handler& sh, int even_odd, scanline_type& sl) {
     /*
@@ -1520,7 +1484,7 @@ public:
   
       for (size_t pno=0; pno<pcount; pno++) {
       
-        const gnash::path &this_path_gnash = paths[pno];
+        const Path<float> &this_path_gnash = paths[pno];
         agg::path_storage &this_path_agg = 
           const_cast<agg::path_storage&>(agg_paths[pno]);
         agg::conv_curve< agg::path_storage > curve(this_path_agg);        
@@ -1562,7 +1526,7 @@ public:
 
   // very similar to draw_shape but used for generating masks. There are no
   // fill styles nor subshapes and such. Just render plain solid shapes.
-  void draw_mask_shape(const std::vector<path> &paths, int even_odd) {
+  void draw_mask_shape(const std::vector< Path<float> > &paths, int even_odd) {
   
     unsigned int mask_count = m_alpha_mask.size();
     
@@ -1593,7 +1557,7 @@ public:
   
   
   template <class scanline_type>
-  void draw_mask_shape_impl(const std::vector<path> &paths, int even_odd,
+  void draw_mask_shape_impl(const std::vector< Path<float> > &paths, int even_odd,
     scanline_type& sl) {
     
     typedef agg::pixfmt_gray8 pixfmt;
@@ -1634,7 +1598,7 @@ public:
 
     for (size_t pno=0, pcount=paths.size(); pno < pcount; pno++) {
     
-      const gnash::path& this_path = paths[pno];
+      const Path<float>& this_path = paths[pno];
       path.remove_all();
       
       // reduce everything to just one fill style!
@@ -1647,7 +1611,7 @@ public:
       unsigned int ecount = this_path.m_edges.size();
       for (unsigned int eno=0; eno<ecount; eno++) {
       
-        const edge &this_edge = this_path.m_edges[eno];
+        const Edge<float> &this_edge = this_path.m_edges[eno];
 
         if (this_edge.is_straight())
           path.line_to(this_edge.ap.x, this_edge.ap.y);
@@ -1673,7 +1637,7 @@ public:
 
 
   /// Just like draw_shapes() except that it draws an outline.
-  void draw_outlines(int subshape_id, const std::vector<path> &paths,
+  void draw_outlines(int subshape_id, const std::vector< Path<float> > &paths,
     const std::vector<agg::path_storage>& agg_paths,
     const std::vector<line_style> &line_styles, const cxform& cx,
     const matrix& linestyle_matrix) {
@@ -1707,7 +1671,7 @@ public:
 
   /// Template for draw_outlines(), see draw_shapes_impl().
   template <class scanline_type>
-  void draw_outlines_impl(int subshape_id, const std::vector<path> &paths,
+  void draw_outlines_impl(int subshape_id, const std::vector< Path<float> > &paths,
     const std::vector<agg::path_storage>& agg_paths,
     const std::vector<line_style> &line_styles, const cxform& cx, 
     const matrix& linestyle_matrix, scanline_type& sl) {
@@ -1749,7 +1713,7 @@ public:
 
       for (size_t pno=0, pcount=paths.size(); pno<pcount; pno++) {
         
-        const gnash::path& this_path_gnash = paths[pno];
+        const Path<float>& this_path_gnash = paths[pno];
         agg::path_storage &this_path_agg = 
           const_cast<agg::path_storage&>(agg_paths[pno]);
         
@@ -1772,14 +1736,39 @@ public:
         
         const line_style& lstyle = line_styles[this_path_gnash.m_line-1];
           
-        int width = lstyle.get_width();
-        if (width==1)
-          stroke.width(1);
+        int thickness = lstyle.getThickness();
+        if (!thickness) stroke.width(1); // hairline
+        else if ( (!lstyle.scaleThicknessVertically()) && (!lstyle.scaleThicknessHorizontally()) )
+        {
+          stroke.width(TWIPS_TO_PIXELS(thickness));
+        }
         else
-          stroke.width(std::max(1.0f, width*stroke_scale));
-          
-        stroke.line_cap(agg::round_cap);        
-        stroke.line_join(agg::round_join); 
+        {
+          if ( (!lstyle.scaleThicknessVertically()) || (!lstyle.scaleThicknessHorizontally()) )
+          {
+             LOG_ONCE( log_unimpl(_("Unidirectionally scaled strokes in AGG renderer (we'll scale by the scalable one)")) );
+          }
+          stroke.width(std::max(1.0f, thickness*stroke_scale));
+        }
+        
+        // TODO: support endCapStyle
+        
+        // TODO: When lstyle.noClose==0 and the start and end point matches,
+        // then render a real join instead of the caps.
+
+        switch (lstyle.startCapStyle()) {
+          case CAP_NONE   : stroke.line_cap(agg::butt_cap); break; 
+          case CAP_SQUARE : stroke.line_cap(agg::square_cap); break;          
+          default : case CAP_ROUND : stroke.line_cap(agg::round_cap); 
+        }
+        
+        switch (lstyle.joinStyle()) {
+          case JOIN_BEVEL : stroke.line_join(agg::bevel_join); break;
+          case JOIN_MITER : stroke.line_join(agg::miter_join); break;
+          default : case JOIN_ROUND : stroke.line_join(agg::round_join);
+        }
+        
+        stroke.miter_limit(lstyle.miterLimitFactor());
                 
         ras.reset();
         ras.add_path(stroke);
@@ -2034,20 +2023,16 @@ public:
   }
   
   void set_scale(float new_xscale, float new_yscale) {
-    /*xscale = new_xscale/20.0f;
-    yscale = new_yscale/20.0f;*/
     
     scale_set=true;
-    
     stage_matrix.set_identity();
     stage_matrix.set_scale(new_xscale/20.0f, new_yscale/20.0f);
   }
 
-  virtual void get_scale(point& scale) {
-    scale.x = PIXELS_TO_TWIPS(stage_matrix.get_x_scale());
-    scale.y = PIXELS_TO_TWIPS(stage_matrix.get_y_scale());
+  void set_translation(float xoff, float yoff) {
+    stage_matrix.set_translation(xoff, yoff);
   }
-  
+
   virtual unsigned int getBytesPerPixel() const {
     return bpp/8;
   }  
@@ -2111,9 +2096,9 @@ DSOEXPORT render_handler_agg_base*  create_render_handler_agg(const char *pixelf
   if (!pixelformat) return NULL;
 
   if (is_little_endian_host())
-    log_debug("framebuffer pixel format is %s (little-endian host)", pixelformat);
+    log_debug(_("Framebuffer pixel format is %s (little-endian host)"), pixelformat);
   else
-    log_debug("framebuffer pixel format is %s (big-endian host)", pixelformat);
+    log_debug(_("Framebuffer pixel format is %s (big-endian host)"), pixelformat);
   
 #ifdef PIXELFORMAT_RGB555  
   if (!strcmp(pixelformat, "RGB555"))
@@ -2171,11 +2156,16 @@ DSOEXPORT const char *agg_detect_pixel_format(unsigned int rofs, unsigned int rs
   unsigned int bofs, unsigned int bsize,
   unsigned int bpp) {
   
-  if (!is_little_endian_host()) {
+  if (!is_little_endian_host() && (bpp>=24)) {
   
     // Swap bits for big endian hosts, because the following tests assume
     // little endians. The pixel format string matches the bytes in memory.
     
+    // This applies for 24 bpp and 32 bpp modes only because AGG uses arrays
+    // in the premultiply() implementation for these modes. 16 bpp modes 
+    // instead use bit shifting, which is transparent to host endianess.
+    // See bug #22799.
+    
     rofs = bpp - rofs - rsize;
     gofs = bpp - gofs - gsize;
     bofs = bpp - bofs - bsize; 
