@@ -43,6 +43,7 @@
 #include <boost/scoped_array.hpp>
 #include "render_handler.h"
 #include "image.h"
+#include <cmath>
 
 namespace gnash {
 
@@ -320,33 +321,90 @@ public:
                                 c.m_b / 255.0, c.m_a / 255.0);
   }
 
+  void set_invalidated_regions(const InvalidatedRanges& ranges)
+  {
+    _invalidated_ranges = ranges;
+  }
+
+  static void
+  snap_to_pixel(cairo_t* cr, double& x, double& y, bool up)
+  {
+    cairo_user_to_device(cr, &x, &y);
+   
+    if (up) {
+      x = std::ceil(x);
+      y = std::ceil(y);
+    } else {
+      x = std::floor(x);
+      y = std::floor(y);
+    }
+
+    cairo_device_to_user(cr, &x, &y);
+  }
+
+  static void
+  snap_to_half_pixel(cairo_t* cr, double& x, double& y)
+  {
+    cairo_user_to_device(cr, &x, &y);
+
+    x = std::floor(x + 0.5) + 0.5;
+    y = std::floor(y + 0.5) + 0.5;
+   
+    cairo_device_to_user(cr, &x, &y);
+  }
+
   virtual void  begin_display(
     const rgba& bg_color,
     int viewport_x0, int viewport_y0,
     int viewport_width, int viewport_height,
     float x0, float x1, float y0, float y1)
   {
-
-  
     float display_width  = fabsf(x1 - x0);
     float display_height = fabsf(y1 - y0);
 
     cairo_identity_matrix(_cr);
-    cairo_rectangle(_cr, x0, y0, display_width, display_height);
-    cairo_clip(_cr);
+
+    cairo_save(_cr);
+
+    if (bg_color.m_a) {
+      set_color(bg_color);
+    }
+
     cairo_scale(_cr, viewport_width / display_width,
                      viewport_height / display_height);
     cairo_translate(_cr, x0, y0);
-  
-	  // Clear the background, if background color has alpha > 0.
-	  if (bg_color.m_a) {
-      set_color(bg_color);
-      cairo_paint(_cr);
+
+    for (size_t rno=0; rno < _invalidated_ranges.size(); rno++) {
+    
+      const Range2d<float>& range = _invalidated_ranges.getRange(rno);
+      if (range.isNull()) {
+        continue;
+      }
+      if (range.isWorld()) {
+        cairo_rectangle(_cr, x0, y0, display_width, display_height);
+        break;
+      }
+
+      double x = range.getMinX(),
+             y = range.getMinY(),
+             maxx = range.getMaxX(),
+             maxy = range.getMaxY();
+
+      snap_to_pixel(_cr, x, y, false);
+      snap_to_pixel(_cr, maxx, maxy, true);
+
+      cairo_rectangle(_cr, x, y, maxx - x, maxy - y);
     }
+
+    cairo_clip(_cr);
+
+    // Paint the background color over the clipped region(s).
+    cairo_paint(_cr);
   }
 
   virtual void  end_display()
   {
+    cairo_restore(_cr);
   }
     
   virtual void  draw_line_strip(const void* coords, int vertex_count,
@@ -432,19 +490,23 @@ public:
     _masks.pop_back();
   }
 
-  void add_path(cairo_t* cr, const path& cur_path)
-  {  
-    cairo_move_to(cr, cur_path.ap.x, cur_path.ap.y);
-    
-    float prev_x = cur_path.ap.x,
-          prev_y = cur_path.ap.y;
+  void add_path(cairo_t* cr, const path& cur_path, bool round_to_half = false)
+  {
+    double x = cur_path.ap.x;
+    double y = cur_path.ap.y;
+
+    snap_to_half_pixel(cr, x, y);
+    cairo_move_to(cr, x, y);
     
     for (std::vector<edge>::const_iterator it = cur_path.m_edges.begin(),
          end = cur_path.m_edges.end(); it != end; ++it) {
       const edge& cur_edge = *it;
       
       if (cur_edge.is_straight()) {
-        cairo_line_to(cr, cur_edge.ap.x, cur_edge.ap.y);
+        x = cur_edge.ap.x;
+        y = cur_edge.ap.y;
+        snap_to_half_pixel(cr, x, y);
+        cairo_line_to(cr, x, y);
       } else {
       
         // Cairo expects a cubic Bezier curve, while Flash gives us a
@@ -453,24 +515,22 @@ public:
         const float two_thirds = 2.0/3.0;
         const float one_third = 1 - two_thirds;
         
-        float x1 = prev_x + two_thirds * (cur_edge.cp.x - prev_x);
-        float y1 = prev_y + two_thirds * (cur_edge.cp.y - prev_y);
+        double x1 = x + two_thirds * (cur_edge.cp.x - x);
+        double y1 = y + two_thirds * (cur_edge.cp.y - y);
         
-        float x2 = cur_edge.cp.x + one_third * (cur_edge.ap.x - cur_edge.cp.x);
-        float y2 = cur_edge.cp.y + one_third * (cur_edge.ap.y - cur_edge.cp.y);
+        double x2 = cur_edge.cp.x + one_third * (cur_edge.ap.x - cur_edge.cp.x);
+        double y2 = cur_edge.cp.y + one_third * (cur_edge.ap.y - cur_edge.cp.y);
         
-        const float& x3 = cur_edge.ap.x;
-        const float& y3 = cur_edge.ap.y;
+        x = cur_edge.ap.x;
+        y = cur_edge.ap.y;
+
+        snap_to_half_pixel(cr, x1, y1);
+        snap_to_half_pixel(cr, x2, y2);
+        snap_to_half_pixel(cr, x, y);    
     
-    
-        cairo_curve_to(cr, x1, y1, x2, y2, x3, y3);
+        cairo_curve_to(cr, x1, y1, x2, y2, x, y);
       }
-      
-      prev_x = cur_edge.ap.x;
-      prev_y = cur_edge.ap.y;
-      
     }
-  
   }
   
   
@@ -496,12 +556,22 @@ public:
   
   void apply_line_style(const line_style& style, const cxform& cx)
   {
-    float width = style.get_width();
+    float width = style.getThickness();
 
     if ( width == 0.0 ) {
-      // TODO: test this!
-      cairo_set_line_width(_cr, 1.0); // expected: 1 pixel
+      double hwidth = 1.0;
+
+      cairo_device_to_user_distance(_cr, &hwidth, &hwidth);
+      cairo_set_line_width(_cr, hwidth);
     } else {
+      // TODO: this is correct for !style.scaleThicknessVertically() 
+      //       and !style.scaleThicknessHorizontally().
+      //       If that's not the case, we should scale the thickness
+      //       togheter with the shapes.
+      if ( style.scaleThicknessVertically() || style.scaleThicknessHorizontally() )
+      {
+        LOG_ONCE( log_unimpl(_("Scaled strokes in Cairo renderer")) );
+      }
       cairo_set_line_width(_cr, width);
     }
     
@@ -645,7 +715,7 @@ public:
       }
       
       apply_line_style(line_styles[cur_path.m_line-1], cx);
-      
+
       add_path(_cr, cur_path);
       
       cairo_stroke(_cr);
@@ -965,6 +1035,7 @@ private:
   std::vector<PathVec> _masks;
   size_t _video_bufsize;
   bool _drawing_mask;
+  InvalidatedRanges _invalidated_ranges;
     
 }; // class render_handler_cairo
 
