@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,16 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: aqua_glue.h,v 1.8 2007/07/24 00:04:39 nihilus Exp $ */
 
 #ifndef AQUA_GLUE_H
 #define AQUA_GLUE_H
 
 #include "gnash.h"
 #include "snappingrange.h"
+#include <AGL/agl.h>
 
 namespace gnash
 {
@@ -34,7 +31,7 @@ class AquaGlue
   public:
     virtual ~AquaGlue() { }
     virtual bool init(int argc, char **argv[]) = 0;
-    virtual bool prepDrawingArea(int width, int height) = 0;
+    virtual bool prepDrawingArea(int width, int height, AGLDrawable drawable) = 0;
     virtual render_handler* createRenderHandler() = 0;
     virtual void render() = 0;
   protected:
@@ -43,4 +40,4 @@ class AquaGlue
 
 }
 
-#endif /* AQUA_GLUE_H */
\ No newline at end of file
+#endif /* AQUA_GLUE_H */
