@@ -15,7 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: sdlsup.h,v 1.24 2008/01/21 20:55:43 rsavoye Exp $ */
 
 #ifndef __SDLSUP_H__
 #define __SDLSUP_H__
@@ -24,7 +23,6 @@
 #include "gnashconfig.h"
 #endif
 
-#include "tu_config.h"
 #include "gui.h"
 
 #include "SDL.h"
@@ -46,7 +44,7 @@
 namespace gnash
 {
 
-class DSOEXPORT SDLGui : public Gui
+class SDLGui : public Gui
 {
 public:
     SDLGui(unsigned long xid, float scale, bool loop, unsigned int depth);
