@@ -17,7 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: curl_adapter.cpp,v 1.47 2008/02/19 19:20:50 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -662,7 +661,7 @@ CurlStreamFile::seek(size_t pos)
 
 	if ( _cached < pos )
 	{
-		fprintf(stderr, "Warning: could not cache anough bytes on seek\n");
+		//fprintf(stderr, "Warning: could not cache anough bytes on seek\n");
 		return false; // couldn't cache so many bytes
 	}
 
