@@ -9,15 +9,12 @@
 #ifndef TU_FILE_H
 #define TU_FILE_H
 
-
-#include "tu_config.h"
-#include <cstdio>
-#include "tu_types.h"
-#include "tu_swap.h"
+#include "dsodefs.h" // DSOEXPORT
 #include "utility.h"
 
-class membuf;
-struct SDL_RWops;
+#include <cstdio>
+
+//class membuf;
 
 enum
 {
@@ -63,21 +60,10 @@ public:
     // Make a file from an ordinary FILE*.
     tu_file(FILE* fp, bool autoclose);
     
-    // Optional: if you're using SDL, this is a constructor to create
-    // a tu_file from an SDL_RWops* stream.
-    tu_file(SDL_RWops* sdl_stream, bool autoclose);
-    
     // Open a file using ordinary fopen().  Automatically closes the
     // file when we are destroyed.
     tu_file(const char* name, const char* mode);
-    
-    // Make a memory-buffer file for read/write.
-    enum memory_buffer_enum { memory_buffer };
-    tu_file(memory_buffer_enum m);
-    
-    // A read-only memory-buffer with predefined data.
-    tu_file(memory_buffer_enum m, int size, void* data);
-    
+   
     ~tu_file();
     
     /// Copy remaining contents of *in into *this.
@@ -86,13 +72,6 @@ public:
     ///       is in error condition, see get_error().
     ///
     void copy_from(tu_file* in);
-
-    /// Copy remaining contents of *this into *out.
-    //
-    /// TODO: define what happens when the stream
-    ///       is in error condition, see get_error().
-    ///
-    void copy_to(membuf* out);
     
     /// Copy a fixed number of bytes from *in to *this.
     //
@@ -110,23 +89,36 @@ public:
     /// TODO: define what happens when the stream
     ///       is in error condition, see get_error().
     ///
-    uint32_t 	read_le32() {
-	// read8() is uint8_t, so no masks with 0xff are required.
-	uint32_t result = (uint32_t)read8();
-	result |= (uint32_t)read8() << 8;
-	result |= (uint32_t)read8() << 16;
-	result |= (uint32_t)read8() << 24;
+    boost::uint32_t 	read_le32() {
+	// read8() is boost::uint8_t, so no masks with 0xff are required.
+	boost::uint32_t result = (boost::uint32_t)read8();
+	result |= (boost::uint32_t)read8() << 8;
+	result |= (boost::uint32_t)read8() << 16;
+	result |= (boost::uint32_t)read8() << 24;
 	return(result);
     }
 	
+	/// \brief Read a 64-bit word from a little-ending stream,
+	/// returning it as a native-endian word.
+	//
+	/// TODO: define what happens when the stream is in
+	///       error condition, see get_error().
+	/// TODO: define a platform-neutral type for 64 bits.
+	long double read_le_double64() {
+		return static_cast<long double> (
+			static_cast<boost::int64_t> (read_le32()) |
+			static_cast<boost::int64_t> (read_le32()) << 32
+		);
+	}
+
     /// \brief Read a 16-bit word from a little-endian stream.
     //
     /// TODO: define what happens when the stream
     ///       is in error condition, see get_error().
     ///
-    uint16_t 	read_le16() {
-	uint16_t result = (uint16_t)read8();
-	result |= (uint16_t)read8() << 8;
+    boost::uint16_t 	read_le16() {
+	boost::uint16_t result = (boost::uint16_t)read8();
+	result |= (boost::uint16_t)read8() << 8;
 	return(result);
     }
 
@@ -135,11 +127,11 @@ public:
     /// TODO: define what happens when the stream
     ///       is in error condition, see get_error().
     ///
-    void 	write_le32(uint32_t u) {
-	write8((uint8_t)u);
-	write8((uint8_t)(u>>8));
-	write8((uint8_t)(u>>16));
-	write8((uint8_t)(u>>24));
+    void 	write_le32(boost::uint32_t u) {
+	write8((boost::uint8_t)u);
+	write8((boost::uint8_t)(u>>8));
+	write8((boost::uint8_t)(u>>16));
+	write8((boost::uint8_t)(u>>24));
     }
 
     /// \brief Write a 16-bit word to a little-endian stream.
@@ -147,9 +139,9 @@ public:
     /// TODO: define what happens when the stream
     ///       is in error condition, see get_error().
     ///
-    void 	write_le16(uint16_t u) {
-	write8((uint8_t)u);
-	write8((uint8_t)(u>>8));
+    void 	write_le16(boost::uint16_t u) {
+	write8((boost::uint8_t)u);
+	write8((boost::uint8_t)(u>>8));
     }
     
     /// \brief Read a single byte from the stream
@@ -157,14 +149,14 @@ public:
     /// TODO: define what happens when the stream
     ///       is in error condition, see get_error().
     ///
-    uint8_t 	read_byte() { return read8(); }
+    boost::uint8_t 	read_byte() { return read8(); }
 
     /// \brief write a single byte to the stream
     //
     /// TODO: define what happens when the stream
     ///       is in error condition, see get_error().
     ///
-    void	write_byte(uint8_t u) { write8(u); }
+    void	write_byte(boost::uint8_t u) { write8(u); }
     
     /// \brief Read the given number of bytes from the stream
     //
@@ -256,46 +248,43 @@ public:
 
     /// \brief Get the size of the stream
     int get_size() { return m_get_stream_size(m_data); }
-
-    // \brief printf-style convenience function.
-    int	printf(const char* fmt, ...);
     
     // \brief UNSAFE back door, for testing only.
     void*	get_app_data_DEBUG() { return m_data; }
     
     
 private:
-    uint64_t	read64() {
-	uint64_t u;
+    boost::uint64_t	read64() {
+	boost::uint64_t u;
 	m_read(&u, 8, m_data);
 	return u;
     }
-    uint32_t	read32() {
-	uint32_t u;
+    boost::uint32_t	read32() {
+	boost::uint32_t u;
 	m_read(&u, 4, m_data);
 	return u;
     }
-    uint16_t	read16() {
-	uint16_t u;
+    boost::uint16_t	read16() {
+	boost::uint16_t u;
 	m_read(&u, 2, m_data);
 	return u;
     }
-    uint8_t	read8() {
-	uint8_t u;
+    boost::uint8_t	read8() {
+	boost::uint8_t u;
 	m_read(&u, 1, m_data);
 	return u;
     }
     
-    void	write64(uint64_t u) {
+    void	write64(boost::uint64_t u) {
 	m_write(&u, 8, m_data);
     }
-    void	write32(uint32_t u) {
+    void	write32(boost::uint32_t u) {
 	m_write(&u, 4, m_data);
     }
-    void	write16(uint16_t u) {
+    void	write16(boost::uint16_t u) {
 	m_write(&u, 2, m_data);
     }
-    void	write8(uint8_t u) {
+    void	write8(boost::uint8_t u) {
 	m_write(&u, 1, m_data);
     }
     
@@ -326,9 +315,9 @@ inline void	tu_file::write_float32(float value)
 {
     union alias {
 	float	f;
-	uint32_t	i;
+	boost::uint32_t	i;
     } u;
-    compiler_assert(sizeof(alias) == sizeof(uint32_t));
+    compiler_assert(sizeof(alias) == sizeof(boost::uint32_t));
     
     u.f = value;
     write_le32(u.i);
@@ -343,7 +332,7 @@ inline float	tu_file::read_float32()
 {
     union {
 	float	f;
-	uint32_t	i;
+	boost::uint32_t	i;
     } u;
     compiler_assert(sizeof(u) == sizeof(u.i));
     
