@@ -8,7 +8,7 @@
 #define ZLIB_ADAPTER_H
 
 
-#include "tu_config.h"
+#include "dsodefs.h"
 
 #include <memory>
 
