@@ -20,15 +20,10 @@
 //
 
 
-/* $Id: Range2d.h,v 1.20 2008/01/21 20:56:11 rsavoye Exp $ */
 
 #ifndef GNASH_RANGE2D_H
 #define GNASH_RANGE2D_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <ostream>
 #include <limits>
 #include <algorithm>
@@ -256,7 +251,8 @@ public:
 	/// Note that WORLD rectangles contain every point
 	/// and NULL rectangles contain no point.
 	///
-	bool contains(T x, T y) const
+	template <typename U>
+	bool contains(U x, U y) const
 	{
 		if ( isNull() ) return false;
 		if ( isWorld() ) return true;
