@@ -20,7 +20,6 @@
 // Based on sound_handler_sdl.cpp by Thatcher Ulrich http://tulrich.com 2003
 // which has been donated to the Public Domain.
 
-// $Id: sound_handler_sdl.cpp,v 1.1.2.1 2008/02/22 14:16:36 strk Exp $
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -45,13 +44,40 @@
 #include <boost/scoped_array.hpp>
 #include <SDL.h>
 
+namespace { // anonymous
+
+// Header of a wave file
+// http://ftp.iptel.org/pub/sems/doc/full/current/wav__hdr_8c-source.html
+typedef struct{
+     char rID[4];            // 'RIFF'
+     long int rLen;        
+     char wID[4];            // 'WAVE'
+     char fId[4];            // 'fmt '
+     long int pcm_header_len;   // varies...
+     short int wFormatTag;
+     short int nChannels;      // 1,2 for stereo data is (l,r) pairs
+     long int nSamplesPerSec;
+     long int nAvgBytesPerSec;
+     short int nBlockAlign;      
+     short int nBitsPerSample;
+} WAV_HDR;
+
+// Chunk of wave file
+// http://ftp.iptel.org/pub/sems/doc/full/current/wav__hdr_8c-source.html
+typedef struct{
+    char dId[4];            // 'data' or 'fact'
+    long int dLen;
+} CHUNK_HDR;
+
+} // end of anonymous namespace
+
 namespace gnash {
 namespace media {
 
-SDL_sound_handler::SDL_sound_handler()
+SDL_sound_handler::SDL_sound_handler(const std::string& wavefile)
 	: soundOpened(false),
-		soundsPlaying(0),
-		muted(false)
+	  soundsPlaying(0),
+	  muted(false)
 {
 	// This is our sound settings
 	audioSpec.freq = 44100;
@@ -60,6 +86,27 @@ SDL_sound_handler::SDL_sound_handler()
 	audioSpec.callback = SDL_sound_handler::sdl_audio_callback;
 	audioSpec.userdata = this;
 	audioSpec.samples = 2048;		//512 - not enough for  videostream
+
+	if (! wavefile.empty() ) {
+        file_stream.open(wavefile.c_str());
+        if (file_stream.fail()) {
+            std::cerr << "Unable to write file '" << wavefile << std::endl;
+            exit(1);
+	    } else {
+                write_wave_header(file_stream);
+                std::cout << "# Created 44100 16Mhz stereo wave file:" << std::endl <<
+                    "AUDIOFILE=" << wavefile << std::endl;
+        }
+    }
+
+}
+
+SDL_sound_handler::SDL_sound_handler()
+	:
+	soundOpened(false),
+	soundsPlaying(0),
+	muted(false)
+{
 }
 
 void
@@ -85,6 +132,8 @@ SDL_sound_handler::~SDL_sound_handler()
 {
 	delete_all_sounds();
 	if (soundOpened) SDL_CloseAudio();
+	if (file_stream) file_stream.close();
+
 }
 
 
@@ -289,24 +338,19 @@ void	SDL_sound_handler::stop_sound(int sound_handle)
 	if (sound_handle < 0 || (unsigned int) sound_handle >= m_sound_data.size())
 	{
 		// Invalid handle.
-	} else {
-	
-		sound_data* sounddata = m_sound_data[sound_handle];
-	
-		for (boost::int32_t i = (boost::int32_t) sounddata->m_active_sounds.size()-1; i >-1; i--) {
-
-			//active_sound* sound = sounddata->m_active_sounds[i];
-
-			// Stop sound, remove it from the active list
-			//sound->delete_raw_data();
-			//delete sound->decoder;
-			sounddata->m_active_sounds.erase(sounddata->m_active_sounds.begin() + i);
-			
-			--soundsPlaying;
-			++_soundsStopped;
-		}
+		return;
 	}
 
+	
+	sound_data* sounddata = m_sound_data[sound_handle];
+
+	size_t nActiveSounds = sounddata->m_active_sounds.size();
+
+	soundsPlaying -= nActiveSounds;
+	_soundsStopped += nActiveSounds;
+
+	sounddata->clearActiveSounds();
+
 }
 
 
@@ -330,21 +374,15 @@ void	SDL_sound_handler::stop_all_sounds()
 {
 	boost::mutex::scoped_lock lock(_mutex);
 
-	boost::int32_t num_sounds = (boost::int32_t) m_sound_data.size()-1;
-	for (boost::int32_t j = num_sounds; j > -1; j--) {//Optimized
-		sound_data* sounddata = m_sound_data[j];
-		boost::int32_t num_active_sounds = (boost::int32_t) sounddata->m_active_sounds.size()-1;
-		for (boost::int32_t i = num_active_sounds; i > -1; i--) {
+	for (Sounds::iterator i=m_sound_data.begin(), e=m_sound_data.end(); i!=e; ++i)
+	{
+		sound_data* sounddata = *i;
+		size_t nActiveSounds = sounddata->m_active_sounds.size();
 
-			//active_sound* sound = sounddata->m_active_sounds[i];
+		soundsPlaying -= nActiveSounds;
+		_soundsStopped += nActiveSounds;
 
-			// Stop sound, remove it from the active list
-			//delete sound->decoder;
-			sounddata->m_active_sounds.erase(sounddata->m_active_sounds.begin() + i);
-			
-			--soundsPlaying;
-			++_soundsStopped;
-		}
+		sounddata->clearActiveSounds();
 	}
 }
 
@@ -514,6 +552,13 @@ create_sound_handler_sdl()
 	return new SDL_sound_handler;
 }
 
+sound_handler*
+create_sound_handler_sdl(const std::string& wave_file)
+// Factory.
+{
+	return new SDL_sound_handler(wave_file);
+}
+
 // Pointer handling and checking functions
 boost::uint8_t*
 active_sound::get_raw_data_ptr(unsigned long int pos)
@@ -630,6 +675,52 @@ do_mixing(Uint8* stream, active_sound* sound, Uint8* data, unsigned int mix_leng
 }
 
 
+// write a wave header, using the current audioSpec settings
+void SDL_sound_handler::write_wave_header(std::ofstream& outfile)
+{
+
+  int i;
+  char obuff[80];
+ 
+  // allocate wav header
+  WAV_HDR wav;
+  CHUNK_HDR chk;
+ 
+  // setup wav header
+  sprintf(obuff,"RIFF");
+  for(i=0;i<4;i++) wav.rID[i] = obuff[i];
+ 
+  sprintf(obuff,"WAVE");
+  for(i=0;i<4;i++) wav.wID[i] = obuff[i];
+  
+  sprintf(obuff,"fmt ");
+  for(i=0;i<4;i++) wav.fId[i] = obuff[i];
+ 
+  wav.nBitsPerSample = ((audioSpec.format == AUDIO_S16SYS) ? 16 : 0);
+  wav.nSamplesPerSec = audioSpec.freq;
+  wav.nAvgBytesPerSec = audioSpec.freq;
+  wav.nAvgBytesPerSec *= wav.nBitsPerSample / 8;
+  wav.nAvgBytesPerSec *= audioSpec.channels;
+  wav.nChannels = audioSpec.channels;
+    
+  wav.pcm_header_len = 16;
+  wav.wFormatTag = 1;
+  wav.rLen = sizeof(WAV_HDR) + sizeof(CHUNK_HDR);
+  wav.nBlockAlign = audioSpec.channels * wav.nBitsPerSample / 8;
+
+  // setup chunk header
+  sprintf(obuff,"data");
+  for(i=0;i<4;i++) chk.dId[i] = obuff[i];
+  chk.dLen = 0;
+ 
+  /* write riff/wav header */
+  outfile.write((char *)&wav, sizeof(WAV_HDR));
+ 
+  /* write chunk header */
+  outfile.write((char *)&chk, sizeof(CHUNK_HDR));
+ 
+}
+
 // Callback invoked by the SDL audio thread.
 void SDL_sound_handler::sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 {
@@ -653,6 +744,7 @@ void SDL_sound_handler::sdl_audio_callback (void *udata, Uint8 *stream, int buff
 	// If nothing to play there is no reason to play
 	// Is this a potential deadlock problem?
 	if (handler->soundsPlaying == 0 && handler->m_aux_streamer.size() == 0) {
+            std::cout << "Pausing Audio now...\n";
 		SDL_PauseAudio(1);
 		return;
 	}
@@ -815,8 +907,30 @@ void SDL_sound_handler::sdl_audio_callback (void *udata, Uint8 *stream, int buff
 		} // active sounds loop
 	} // existing sounds loop
 
+	// 
+	// WRITE CONTENTS OF stream TO FILE
+	//
+	if (handler->file_stream) {
+            handler->file_stream.write((char*) stream, buffer_length_in);
+            // now, mute all audio
+            memset ((void*) stream, 0, buffer_length_in);
+	}
+
+
+
 }
 
+void
+sound_data::clearActiveSounds()
+{
+	for (ActiveSounds::iterator i=m_active_sounds.begin(), e=m_active_sounds.end(); i!=e; ++i)
+	{
+		delete *i;
+	}
+	m_active_sounds.clear();
+}
+
+
 } // gnash.media namespace 
 } // namespace gnash
 
