@@ -18,10 +18,8 @@
 #ifndef GNASH_AS_ENVIRONMENT_H
 #define GNASH_AS_ENVIRONMENT_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
+#include "smart_ptr.h" // GNASH_USE_GC
+#include "dsodefs.h" // DSOEXPORT
 #include "as_value.h" // for composition (vector + frame_slot)
 #include "StringPredicates.h" // for Variables 
 #include "as_object.h"
@@ -345,16 +343,15 @@ public:
 	as_object* find_object(const std::string& path, const ScopeStack* scopeStack=NULL) const;
 
 	/// Dump content of the stack to a std::ostream
-	void dump_stack(std::ostream& out=std::cerr)
-	{
-		out << "Stack: ";
-		for (unsigned int i=0, n=m_stack.size(); i<n; i++)
-		{
-			if (i) out << " | ";
-			out << '"' << m_stack[i] << '"';
-		}
-		out << std::endl;
-	}
+	//
+	/// @param out
+	///	The output stream, standard error if omitted.
+	///
+	/// @param limit
+	///	If > 0, limit number of printed item by the given amount (from the top).
+	///	Unlimited by default;
+	///
+	void dump_stack(std::ostream& out=std::cerr, unsigned int limit=0) const;
 
 	/// Dump the local registers to a std::ostream
 	//
@@ -477,7 +474,7 @@ public:
 	}
 
 	/// Return the depth of call stack
-	size_t callStackDepth()
+	static size_t callStackDepth()
 	{
 		return _localFrames.size();
 	}
@@ -509,13 +506,13 @@ private:
 	/// @param func
 	///	The function being called
 	///
-	static void pushCallFrame(as_function* func);
+	static DSOEXPORT void pushCallFrame(as_function* func);
 
 	/// Remove current call frame from the stack
 	//
 	/// This should happen when an ActionScript function returns.
 	///
-	static void popCallFrame();
+	static DSOEXPORT void popCallFrame();
 	
 	/// Return the (possibly UNDEFINED) value of the named variable.
 	//
