@@ -17,10 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "as_value.h"
 #include "as_object.h"
 #include "as_function.h" // for as_function
@@ -47,9 +44,7 @@
 #	define snprintf _snprintf
 #endif
 
-#ifdef HAVE_FINITE
-#define isfinite finite
-#endif
+using namespace std;
 
 // Define the macro below to make abstract equality operator verbose
 //#define GNASH_DEBUG_EQUALITY 1
@@ -269,6 +264,29 @@ as_value::to_primitive() const
 	return to_primitive(hint);
 }
 
+as_value&
+as_value::convert_to_primitive()
+{
+	VM& vm = VM::get();
+	int swfVersion = vm.getSWFVersion();
+
+	type hint = NUMBER;
+
+	if ( m_type == OBJECT && swfVersion > 5 && getObj()->isDateObject() )
+	{
+		hint = STRING;
+	}
+
+#if 0
+	else if ( m_type == MOVIECLIP && swfVersion > 5 )
+	{
+		throw ActionTypeError();
+	}
+#endif
+
+	return convert_to_primitive(hint);
+}
+
 // Conversion to primitive value.
 as_value
 as_value::to_primitive(type hint) const
@@ -296,18 +314,25 @@ as_value::to_primitive(type hint) const
 		if ( m_type == OBJECT ) obj = getObj().get();
 		else obj = getFun().get();
 
-		if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) || (!method.is_function()) ) // ECMA says ! is_object()
+		if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) || (!method.is_object()) ) // ECMA says ! is_object()
 		{
 #if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
 			log_debug(" valueOf not found");
 #endif
-			if ( (!obj->get_member(NSV::PROP_TO_STRING, &method)) || (!method.is_function()) ) // ECMA says ! is_object()
-			{
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-				log_debug(" toString not found");
-#endif
-				throw ActionTypeError();
-			}
+            // Returning undefined here instead of throwing
+            // a TypeError passes tests in actionscript.all/Object.as
+            // and many swfdec tests, with no new failures (though
+            // perhaps we aren't testing enough).
+            return as_value();
+            
+//			if ( (!obj->get_member(NSV::PROP_TO_STRING, &method)) || (!method.is_object()) ) // ECMA says ! is_object()
+//			{
+//#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+//				log_debug(" toString not found");
+//#endif
+//                
+//				throw ActionTypeError();
+//			}
 		}
 	}
 	else
@@ -373,6 +398,108 @@ as_value::to_primitive(type hint) const
 
 }
 
+// Conversion to primitive value.
+as_value&
+as_value::convert_to_primitive(type hint) 
+{
+	if ( m_type != OBJECT && m_type != AS_FUNCTION ) return *this; 
+	//if ( ! is_object() ) return *this; // include MOVIECLIP !!
+
+#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+	log_debug("to_primitive(%s)", hint==NUMBER ? "NUMBER" : "STRING");
+#endif 
+
+	// TODO: implement as_object::DefaultValue (ECMA-262 - 8.6.2.6)
+
+	as_value method;
+	as_object* obj = NULL;
+
+	if (hint == NUMBER)
+	{
+		if ( m_type == MOVIECLIP )
+		{
+			set_double(NAN);
+			return *this;
+		}
+		if ( m_type == OBJECT ) obj = getObj().get();
+		else obj = getFun().get();
+
+		if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) || (!method.is_object()) ) // ECMA says ! is_object()
+		{
+#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+			log_debug(" valueOf not found");
+#endif
+			// Returning undefined here instead of throwing
+			// a TypeError passes tests in actionscript.all/Object.as
+			// and many swfdec tests, with no new failures (though
+			// perhaps we aren't testing enough).
+			set_undefined();
+			return *this;
+            
+		}
+	}
+	else
+	{
+		assert(hint==STRING);
+
+		if ( m_type == MOVIECLIP )
+		{
+			set_string(getCharacterProxy().getTarget());
+			return *this;
+		}
+
+		if ( m_type == OBJECT ) obj = getObj().get();
+		else obj = getFun().get();
+
+		// @@ Moock says, "the value that results from
+		// calling toString() on the object".
+		//
+		// When the toString() method doesn't exist, or
+		// doesn't return a valid number, the default
+		// text representation for that object is used
+		// instead.
+		//
+		if ( ! obj->useCustomToString() )
+		{
+#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+			log_debug(" not using custom toString");
+#endif
+			set_string(obj->get_text_value());
+			return *this;
+		}
+
+		if ( (!obj->get_member(NSV::PROP_TO_STRING, &method)) || (!method.is_function()) ) // ECMA says ! is_object()
+		{
+#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+			log_debug(" toString not found");
+#endif
+			if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) || (!method.is_function()) ) // ECMA says ! is_object()
+			{
+#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+				log_debug(" valueOf not found");
+#endif
+				throw ActionTypeError();
+			}
+		}
+	}
+
+	assert(obj);
+
+	as_environment env;
+	as_value ret = call_method0(method, &env, obj);
+#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
+	log_debug("to_primitive: method call returned %s", ret.to_debug_string().c_str());
+#endif
+	if ( ret.m_type == OBJECT || ret.m_type == AS_FUNCTION ) // not a primitive 
+	{
+		throw ActionTypeError();
+	}
+
+	*this = ret;
+
+	return *this;
+}
+
 double
 as_value::to_number() const
 {
@@ -399,10 +526,27 @@ as_value::to_number() const
 			
 		}
             }
+            else if(swfversion <= 4)
+            {
+                // For SWF4, any valid number before non-numerical
+                // characters is returned, including exponent, positive
+                // and negative signs and whitespace before. A stringstream
+                // does the job. Locale differences (decimal separator) arise
+                // with strtod.                
+                double d = 0;
+                std::istringstream is (getStr());
+                is >> d;
+                return d;
+            }
 
             // @@ Moock says the rule here is: if the
             // string is a valid float literal, then it
             // gets converted; otherwise it is set to NaN.
+            // Valid for SWF5 and above.
+            //
+            // boost::lexical_cast is remarkably inflexible and 
+            // fails for anything that has non-numerical characters.
+            // Fortunately, actionscript is equally inflexible.
             try 
             { 
                 double d = boost::lexical_cast<double>(getStr());
@@ -414,18 +558,19 @@ as_value::to_number() const
             // just like for any other non-numerical text. This is correct
             // behaviour.
             {
-            	if(swfversion <= 4) return (double)0.0;
-            	else return (double)NAN;
+            	return (double)NAN;
             }
         }
 
         case NULLTYPE:
-        case UNDEFINED:
+        case UNDEFINED: 
+	{
             // Evan: from my tests
             // Martin: FlashPlayer6 gives 0; FP9 gives NaN.
             return ( swfversion >= 7 ? NAN : 0 );
+	}
 
-        case BOOLEAN:
+        case BOOLEAN: 
             // Evan: from my tests
             // Martin: confirmed
             return getBool() ? 1 : 0;
@@ -466,9 +611,11 @@ as_value::to_number() const
         }
 
         case MOVIECLIP:
+	{
             // This is tested, no valueOf is going
             // to be invoked for movieclips.
             return NAN; 
+	}
 
         default:
             // Other object types should return NaN, but if we implement that,
@@ -478,24 +625,28 @@ as_value::to_number() const
     /* NOTREACHED */
 }
 
+// This returns an as_value as an integer. It is
+// probably used for most implicit conversions to 
+// int, for instance in the String class.
 boost::int32_t
 as_value::to_int() const
 {
 	double d = to_number();
-	int i=0;
 
 	if ( ! isfinite(d) ) return 0;
 
-	if (d < 0)
-	{
-		i = - (boost::uint32_t) fmod (-d, 4294967296.0);
-	}
-	else
-	{
-		i = (boost::uint32_t) fmod (d, 4294967296.0);
-	}
+	boost::int32_t i = 0;
 
-	return i;
+    if (d < 0)
+    {   
+	    i = - static_cast<boost::uint32_t>(fmod (-d, 4294967296.0));
+    }
+    else
+    {
+	    i = static_cast<boost::uint32_t>(fmod (d, 4294967296.0));
+    }
+    
+    return i;
 }
 
 // Conversion to boolean for SWF7 and up
@@ -534,14 +685,9 @@ as_value::to_bool_v5() const
 	    {
 		case  STRING:
 		{
-			if (getStr() == "false") return false;
-			else if (getStr() == "true") return true;
-			else
-			{
-				double num = to_number();
-				bool ret = num && ! isnan(num);
-				return ret;
-			}
+			double num = to_number();
+			bool ret = num && ! isnan(num);
+			return ret;
 		}
 		case NUMBER:
 		{
@@ -571,14 +717,9 @@ as_value::to_bool_v6() const
 	    {
 		case  STRING:
 		{
-			if (getStr() == "false") return false;
-			else if (getStr() == "true") return true;
-			else
-			{
-				double num = to_number();
-				bool ret = num && ! isnan(num);
-				return ret;
-			}
+			double num = to_number();
+			bool ret = num && ! isnan(num);
+			return ret;
 		}
 		case NUMBER:
 		{
@@ -671,7 +812,6 @@ as_value::set_sprite(sprite_instance& sprite)
 void
 as_value::set_character(character& sprite)
 {
-	drop_refs();
 	m_type = MOVIECLIP;
 	_value = CharacterProxy(&sprite);
 }
@@ -708,7 +848,6 @@ void
 as_value::convert_to_string()
 {
     std::string ns = to_string();
-    drop_refs();
     m_type = STRING;	// force type.
     _value = ns;
 }
@@ -719,7 +858,6 @@ as_value::convert_to_string_versioned(int version)
     // Force type to string.
 {
     std::string ns = to_string_versioned(version);
-    drop_refs();
     m_type = STRING;	// force type.
     _value = ns;
 }
@@ -728,7 +866,6 @@ as_value::convert_to_string_versioned(int version)
 void
 as_value::set_undefined()
 {
-	drop_refs();
 	m_type = UNDEFINED;
 	_value = boost::blank();
 }
@@ -736,7 +873,6 @@ as_value::set_undefined()
 void
 as_value::set_null()
 {
-	drop_refs();
 	m_type = NULLTYPE;
 	_value = boost::blank();
 }
@@ -763,7 +899,6 @@ as_value::set_as_object(as_object* obj)
 	}
 	if (m_type != OBJECT || getObj() != obj)
 	{
-		drop_refs();
 		m_type = OBJECT;
 		_value = boost::intrusive_ptr<as_object>(obj);
 	}
@@ -780,7 +915,6 @@ as_value::set_as_function(as_function* func)
 {
     if (m_type != AS_FUNCTION || getFun().get() != func)
     {
-	drop_refs();
 	m_type = AS_FUNCTION;
 	if (func)
 	{
@@ -1034,7 +1168,8 @@ as_value::typeOf() const
 			return "null";
 
 		case as_value::AS_FUNCTION:
-			return "function";
+			if ( getFun()->isSuper() ) return "object";
+			else return "function";
 
 		default:
 			if (is_exception())
@@ -1122,7 +1257,7 @@ as_value::to_debug_string() const
 		case AS_FUNCTION:
 		{
 			as_function* obj = getFun().get();
-			sprintf(buf, "[function:%p]", (void *)obj);
+			sprintf(buf, "[function(%s):%p]", typeName(*obj).c_str(), (void *)obj);
 			return buf;
 		}
 		case STRING:
@@ -1188,9 +1323,6 @@ as_value::operator=(const as_value& v)
 
 as_value::as_value(boost::intrusive_ptr<as_object> obj)
 	:
-	// Initialize to non-object type here,
-	// or set_as_object will call
-	// drop_ref on undefined memory !!
 	m_type(UNDEFINED)
 {
 	set_as_object(obj);
@@ -1419,7 +1551,6 @@ as_value::getSprite(bool allowUnloaded) const
 void
 as_value::set_string(const std::string& str)
 {
-	drop_refs();
 	m_type = STRING;
 	_value = str;
 }
@@ -1427,7 +1558,6 @@ as_value::set_string(const std::string& str)
 void
 as_value::set_double(double val)
 {
-	drop_refs();
 	m_type = NUMBER;
 	_value = val;
 }
@@ -1435,7 +1565,6 @@ as_value::set_double(double val)
 void
 as_value::set_bool(bool val)
 {
-	drop_refs();
 	m_type = BOOLEAN;
 	_value = val;
 }
@@ -1519,17 +1648,14 @@ as_value::as_value(unsigned long val)
 
 as_value::as_value(as_object* obj)
 	:
-	// Initialize to non-object type here,
-	// or set_as_object will call
-	// drop_ref on undefined memory !!
 	m_type(UNDEFINED)
 {
 	set_as_object(obj);
 }
 
-as_value::~as_value()
-{ 
-	drop_refs(); 
+/// Chad: Document this
+as_value::as_value(asNamespace &)
+{
 }
 
 //-------------------------------------
@@ -1574,6 +1700,63 @@ as_value::CharacterProxy::setReachable() const
 	if ( _ptr ) _ptr->setReachable();
 }
 
+as_value&
+as_value::newAdd(const as_value& op2)
+{
+	as_value v2=op2;
+
+	try { convert_to_primitive(); }
+	catch (ActionTypeError& e)
+	{
+		log_debug("%s.to_primitive() threw an error during as_value operator+",
+			to_debug_string().c_str());
+	}
+
+	try { v2 = v2.to_primitive(); }
+	catch (ActionTypeError& e)
+	{
+		log_debug("%s.to_primitive() threw an error during as_value operator+",
+			op2.to_debug_string().c_str());
+	}
+
+#if GNASH_DEBUG
+	log_debug(_("(%s + %s) [primitive conversion done]"),
+			to_debug_string().c_str(),
+			v2.to_debug_string().c_str());
+#endif
+
+	if (is_string() || v2.is_string() )
+	{
+		// use string semantic
+
+		int version = VM::get().getSWFVersion();
+		convert_to_string_versioned(version);
+		string_concat(v2.to_string_versioned(version));
+	}
+	else
+	{
+		// use numeric semantic
+		double v2num = v2.to_number();
+		//log_debug(_("v2 num = %g"), v2num);
+		double v1num = to_number();
+		//log_debug(_("v1 num = %g"), v1num);
+
+		set_double(v2num + v1num); 
+
+	}
+
+	return *this;
+}
+
+as_value&
+as_value::subtract(const as_value& op2)
+{
+	double operand1 = to_number();
+	double operand2 = op2.to_number();
+	set_double(operand1 - operand2);
+	return *this;
+}
+
 } // namespace gnash
 
 
