@@ -15,7 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: as_value.h,v 1.84 2008/02/19 12:06:07 strk Exp $ */
 
 #ifndef GNASH_AS_VALUE_H
 #define GNASH_AS_VALUE_H
@@ -24,7 +23,7 @@
 #include "gnashconfig.h"
 #endif
 
-#include "tu_config.h"
+#include "dsodefs.h"
 #include "smart_ptr.h"
 
 #include <cmath>
@@ -32,6 +31,9 @@
 #include <string>
 #include <boost/variant.hpp>
 #include <ostream> // for inlined output operator
+#include <boost/type_traits/is_floating_point.hpp>
+#include <boost/utility/enable_if.hpp>
+#include <boost/static_assert.hpp>
 
 #include "string_table.h"
 
@@ -60,25 +62,18 @@ class asName;
 #endif
 
 
-#ifndef isnan
-# define isnan(x) \
-(sizeof (x) == sizeof (long double) ? isnan_ld (x) \
-: sizeof (x) == sizeof (double) ? isnan_d (x) \
-: isnan_f (x))
-static inline int isnan_f  (float       x) { return x != x; }
-static inline int isnan_d  (double      x) { return x != x; }
-static inline int isnan_ld (long double x) { return x != x; }
-#endif
-	  
-#ifndef isinf
-# define isinf(x) \
-(sizeof (x) == sizeof (long double) ? isinf_ld (x) \
-: sizeof (x) == sizeof (double) ? isinf_d (x) \
-: isinf_f (x))
-	static inline int isinf_f  (float       x) { return isnan (x - x); }
-	static inline int isinf_d  (double      x) { return isnan (x - x); }
-	static inline int isinf_ld (long double x) { return isnan (x - x); }
-#endif
+// The following template works just like its C counterpart, with added
+// type safety (i.e., they will only compile for floating point arguments).
+
+template <typename T>
+inline bool
+isnan(const T& num, typename boost::enable_if<boost::is_floating_point<T> >::type* dummy = 0)
+{
+	UNUSED(dummy);
+	return num != num;
+}
+
+#define isinf(x) (isnan(x - x))
 
 /// Use this methods to obtain a properly-formatted property name
 /// The methods will convert the name to lowercase if the current VM target
@@ -175,7 +170,7 @@ public:
 	as_value(unsigned long val);
 
 	/// Chad: Document this
-	as_value(asNamespace &) {/**/}
+	as_value(asNamespace &);
 
 	/// Construct a NULL, OBJECT, MOVIECLIP or AS_FUNCTION value
 	//
@@ -192,21 +187,12 @@ public:
 	/// Construct a NULL or AS_FUNCTION value
 	as_value(as_function* func);
 
-	~as_value();
-
 	/// Convert numeric value to string value, following ECMA-262 specification
 	//
 	/// TODO: move here some of the good comments found in the function definition.
 	///
 	static std::string doubleToString(double val, int radix=10);
 
-	/// Drop any ref counts we have.
-	//
-	/// This happens prior to changing our value.
-	/// Useful when changing types/values.
-	///
-	void	drop_refs() {}
-
 	/// Return the primitive type of this value, as a string.
 	const char* typeOf() const;
 
@@ -399,13 +385,19 @@ public:
 	/// or NULL if it is not an ActionScript function.
 	as_function*	to_as_function() const;
 
-	/// Return value as a primitive type
+	/// Convert this value to a primitive type
 	//
 	/// Primitive types are: undefined, null, boolean, string, number.
 	/// See ECMA-2.6.2 (sections 4.3.2 and 8.6.2.6).
 	///
-	/// @param env
-	/// 	The environment to use for calling the valueOf method.
+	/// @throw ActionTypeError if an object can't be converted to a primitive
+	///
+	as_value& convert_to_primitive();
+
+	/// Return value as a primitive type
+	//
+	/// Primitive types are: undefined, null, boolean, string, number.
+	/// See ECMA-2.6.2 (sections 4.3.2 and 8.6.2.6).
 	///
 	/// @throw ActionTypeError if an object can't be converted to a primitive
 	///
@@ -426,6 +418,18 @@ public:
 	///
 	as_value to_primitive(type hint) const;
 
+	/// Convert this value to a primitive type, with a preference
+	//
+	/// Primitive types are: undefined, null, boolean, string, number.
+	/// See ECMA-2.6.2 (sections 4.3.2 and 8.6.2.6).
+	///
+	/// @param hint
+	/// 	NUMBER or STRING, the preferred representation we're asking for.
+	///
+	/// @throw ActionTypeError if an object can't be converted to a primitive
+	///
+	as_value& convert_to_primitive(type hint);
+
 	/// Force type to number.
 	//
 	/// @param env
@@ -564,6 +568,14 @@ public:
 	/// Sets this value to this string plus the given string.
 	void	string_concat(const std::string& str);
 
+	/// Equivalent of ActionNewAdd
+	as_value& newAdd(const as_value& v1);
+
+	// Equivalent of ActionSubtract
+	as_value& subtract(const as_value& o);
+
+
+
 	/// Set any object value as reachable (for the GC)
 	//
 	/// Object values are values stored by pointer (objects and functions)
