@@ -375,7 +375,11 @@ sharedobject_getlocal(const fn_call& fn)
             // TODO: don't fail if the movie url has a component ending with .sol (eh...)
             //
             if (newdir.rfind(".sol") != (newdir.size()-4)) {
+#ifndef _WIN32
                 int ret = mkdir(newdir.c_str(), S_IRUSR|S_IWUSR|S_IXUSR);
+#else
+                int ret = mkdir(newdir.c_str());
+#endif
                 if ((errno != EEXIST) && (ret != 0)) {
                     log_error("Couldn't create directory for .sol files: %s\n\t%s",
                               newdir.c_str(), strerror(errno));
@@ -395,7 +399,7 @@ sharedobject_getlocal(const fn_call& fn)
     
     vector<Element *>::iterator it;
     vector<Element *> els = sol.getElements();
-    log_debug("Read "SIZET_FMT" AMF objects from %s", els.size(), newspec.c_str());
+    log_debug("Read %d AMF objects from %s", els.size(), newspec.c_str());
 
     string_table& st = obj->getVM().getStringTable();
     string_table::key dataKey =  obj->getVM().getStringTable().find("data");
@@ -405,18 +409,18 @@ sharedobject_getlocal(const fn_call& fn)
     for (it = els.begin(); it != els.end(); it++) {
         Element *el = (*(it));
 //        log_debug("Adding \"%s\"", el->name.c_str());
-        if (el->getType() == Element::NUMBER) {
+        if (el->getType() == Element::NUMBER_AMF0) {
             double dub =  *((double *)el->getData());
             ptr->set_member(st.string_table::find(el->getName()), as_value(dub));
         } 
-        if (el->getType() == Element::BOOLEAN) {
+        if (el->getType() == Element::BOOLEAN_AMF0) {
             if (el->to_bool() == true) {
                 ptr->set_member(st.string_table::find(el->getName()), as_value(true));
             } else {
                 ptr->set_member(st.string_table::find(el->getName()), as_value(false));
             }       
         } 
-        if (el->getType() == Element::STRING) {
+        if (el->getType() == Element::STRING_AMF0) {
             if (el->getLength() == 0) {
                 ptr->set_member(st.string_table::find(el->getName()), as_value(""));
             } else {
@@ -424,7 +428,7 @@ sharedobject_getlocal(const fn_call& fn)
                 ptr->set_member(st.string_table::find(el->getName()), as_value(str));
             }
         } 
-        if (el->getType() == Element::OBJECT) {
+        if (el->getType() == Element::OBJECT_AMF0) {
 //            data.convert_to_object();
 //            ptr->set_member(st.string_table::find(el->name), data);
         } 
