@@ -21,6 +21,7 @@
 #include "gnashconfig.h"
 #endif
 
+#include "network.h"
 #include "utility.h"
 #include "xml.h"
 #include "xmlsocket.h"
@@ -259,7 +260,7 @@ XMLSocket::anydata(int fd, MessageList& msgs)
                 *eom = 0;
             }
             msgs.push_back( packet );
-            log_debug(_("%d: Pushing Packet of size " SIZET_FMT " at %p"), __LINE__, strlen(packet), packet);
+            log_debug(_("%d: Pushing Packet of size %d at %p"), __LINE__, strlen(packet), packet);
             processing(false);
             return true;
         }
@@ -596,12 +597,20 @@ attachXMLSocketInterface(as_object& o)
     o.init_member("connect", new builtin_function(xmlsocket_connect));
     o.init_member("send", new builtin_function(xmlsocket_send));
     o.init_member("close", new builtin_function(xmlsocket_close));
+
+
+    // all this crap to satisfy swfdec testsuite... (xml-socket-properties*)
+    as_object* onDataIface = new as_object(getObjectInterface());
+    as_function* onDataFun = new builtin_function(xmlsocket_onData, onDataIface);
+    o.init_member("onData", onDataFun);
+    onDataIface->init_member(NSV::PROP_CONSTRUCTOR, as_value(onDataFun));
 }
 
 static void
-attachXMLSocketProperties(as_object& o)
+attachXMLSocketProperties(as_object& /*o*/)
 {
-    o.init_member("onData", new builtin_function(xmlsocket_onData));
+    // this is attached to proto
+    //o.init_member("onData", new builtin_function(xmlsocket_onData));
 }
 
 // extern (used by Global.cpp)
@@ -651,10 +660,10 @@ xmlsocket_as_object::checkForIncomingData()
     std::vector<std::string > msgs;
     if (obj.anydata(msgs))
     {
-        log_debug(_("Got " SIZET_FMT " messages: "), msgs.size());
+        log_debug(_("Got %d messages: "), msgs.size());
         for (size_t i=0; i<msgs.size(); ++i)
         {
-            log_debug(_(" Message " SIZET_FMT " : %s "), i, msgs[i].c_str());
+            log_debug(_(" Message %d: %s "), i, msgs[i].c_str());
         }
 
         boost::intrusive_ptr<as_function> onDataHandler = getEventHandler("onData");
