@@ -1,6 +1,6 @@
 // character.cpp:  ActionScript Character class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,27 +17,44 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 // 
 
-/* $Id: character.cpp,v 1.47 2007/07/11 00:33:57 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h" // USE_SWFTREE
 #endif
 
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "character.h"
 #include "sprite_instance.h"
 #include "drag_state.h" // for do_mouse_drag (to be moved in movie_root)
 #include "VM.h" // for do_mouse_drag (to be moved in movie_root)
 #include "fn_call.h" // for shared ActionScript getter-setters
 #include "GnashException.h" // for shared ActionScript getter-setters (ensure_character)
+#include "render.h"  // for bounds_in_clipping_area()
 #include "ExecutableCode.h"
+#include "namedStrings.h"
+
+#ifdef USE_SWFTREE
+# include "tree.hh"
+#endif
 
 #include <boost/algorithm/string/case_conv.hpp>
 
 #undef set_invalidated
 
+// This is for std::isfinite
+// FIXME: include the std namespace in the macro, and expose
+//        the macro in a single place !
+using namespace std;
+
 namespace gnash
 {
 
+// Define static const members or there will be linkage problems.
+const int character::staticDepthOffset;
+const int character::removedDepthOffset;
+const int character::noClipDepthValue;
+const int character::dynClipDepthValue;
+
 // Initialize unnamed instance count
 unsigned int character::_lastUnnamedInstanceNum=0;
 
@@ -51,60 +68,6 @@ character::getNextUnnamedInstanceName()
 }
 
 
-// TODO: this should likely go in movie_root instead !
-void
-character::do_mouse_drag()
-{
-	drag_state st;
-	_vm.getRoot().get_drag_state(st);
-	if ( this == st.getCharacter() )
-	{
-		// We're being dragged!
-		int	x, y, buttons;
-		get_root_movie()->get_mouse_state(x, y, buttons);
-
-		point world_mouse(PIXELS_TO_TWIPS(x), PIXELS_TO_TWIPS(y));
-		if ( st.hasBounds() )
-		{
-			// Clamp mouse coords within a defined rect.
-			// (it is assumed that drag_state keeps
-			st.getBounds().clamp(world_mouse);
-		}
-
-		if (st.isLockCentered())
-		{
-		    matrix	world_mat = get_world_matrix();
-		    point	local_mouse;
-		    world_mat.transform_by_inverse(&local_mouse, world_mouse);
-
-		    matrix	parent_world_mat;
-		    if (m_parent != NULL)
-			{
-			    parent_world_mat = m_parent->get_world_matrix();
-			}
-
-		    point	parent_mouse;
-		    parent_world_mat.transform_by_inverse(&parent_mouse, world_mouse);
-					
-		    // Place our origin so that it coincides with the mouse coords
-		    // in our parent frame.
-		    matrix	local = get_matrix();
-		    local.set_translation( parent_mouse.m_x, parent_mouse.m_y );
-		    set_matrix(local);
-		}
-		else
-		{
-			// FIXME: Implement relative drag...
-			static bool warned_relative_drag = false;
-			if ( ! warned_relative_drag )
-			{
-				log_unimpl(_("Relative drag"));
-		    		warned_relative_drag = true;
-		    	}
-		}
-	}
-}
-
 matrix
 character::get_world_matrix() const
 {
@@ -131,16 +94,7 @@ character::get_world_cxform() const
 	return m;
 }
 
-sprite_instance*
-character::get_root_movie()
-{
-	assert(m_parent != NULL);
-#ifndef GNASH_USE_GC
-	assert(m_parent->get_ref_count() > 0);
-#endif // GNASH_USE_GC
-	return m_parent->get_root_movie();
-}
-
+#if 0
 void
 character::get_mouse_state(int& x, int& y, int& buttons)
 {
@@ -150,15 +104,20 @@ character::get_mouse_state(int& x, int& y, int& buttons)
 #endif // GNASH_USE_GC
 	get_parent()->get_mouse_state(x, y, buttons);
 }
+#endif
 
-character*
-character::get_relative_target_common(const std::string& name)
+as_object*
+character::get_path_element_character(string_table::key key)
 {
-	if (name == "." || name == "this")
+	if (key == NSV::PROP_uROOT)
 	{
-	    return this;
+		// getAsRoot() will handle _lockroot 
+		return const_cast<sprite_instance*>(getAsRoot());
 	}
-	else if (name == ".." || name == "_parent")
+
+	const std::string& name = _vm.getStringTable().value(key);
+
+	if (name == ".." || key == NSV::PROP_uPARENT )
 	{
 		// Never NULL
 		character* parent = get_parent();
@@ -170,23 +129,29 @@ character::get_relative_target_common(const std::string& name)
 				" a nonexistent parent with '..' "
 				" (a nonexistent parent probably only "
 				"occurs in the root MovieClip)."
-				" Returning a reference to top parent "
-				"(probably the root clip)."));
+				" Returning NULL. "));
 			);
-			//parent = this;
-			assert(this == get_root_movie());
-			return this;
+			return NULL;
 		}
 		return parent;
 	}
-	else if (name == "_root")
+
+	// TODO: is it correct to check for _level here ?
+	//       would it be valid at all if not the very first element
+	//       in a path ?
+	unsigned int levelno;
+	if ( _vm.getRoot().isLevelTarget(name, levelno) )
 	{
-		return get_root_movie();
+		return _vm.getRoot().getLevel(levelno).get();
 	}
-	else if (name.compare(0, 6, "_level") == 0 && name.find_first_not_of("0123456789", 7) == string::npos )
+
+
+	std::string namei = name;
+	if ( _vm.getSWFVersion() < 7 ) boost::to_lower(namei);
+
+	if (name == "." || namei == "this") 
 	{
-		unsigned int levelno = atoi(name.c_str()+6);
-		return VM::get().getRoot().getLevel(levelno).get();
+	    return this;
 	}
 
 	return NULL;
@@ -201,9 +166,9 @@ character::set_invalidated()
 void
 character::set_invalidated(const char* debug_file, int debug_line)
 {
-  // Set the invalidated-flag of the parent. Note this does not mean that
-  // the parent must re-draw itself, it just means that one of it's childs
-  // needs to be re-drawn.
+	// Set the invalidated-flag of the parent. Note this does not mean that
+	// the parent must re-draw itself, it just means that one of it's childs
+	// needs to be re-drawn.
 	if ( m_parent ) m_parent->set_child_invalidated(); 
   
   
@@ -219,19 +184,17 @@ character::set_invalidated(const char* debug_file, int debug_line)
 		m_invalidated = true;
 		
 		#ifdef DEBUG_SET_INVALIDATED
-		printf("%p set_invalidated() of %s in %s:%d\n", this, get_name().c_str(),
-			debug_file, debug_line);
+		printf("%p set_invalidated() of %s in %s:%d\n",
+			this, get_name(), debug_file, debug_line);
+		#else
+		UNUSED(debug_file);
+		UNUSED(debug_line);
 		#endif
 		
-		// NOTE: we need to set snap_distance in order to avoid too tight 
-		// invalidated ranges. The GUI chooses the appropriate distance in base
-		// of various parameters but for this internal ranges list we don't know
-		// that value. So we set snap_distance to some generic value and hope this
-		// does not produce too many nor too coarse ranges. Note when calculating
-		// the actual invalidated ranges the correct distance is used (but there
-		// may be problems when the GUI chooses a smaller value). Needs to be 
-		// fixed. 
-		m_old_invalidated_ranges.snap_distance = 200.0; 
+		// NOTE: the SnappingRanges instance used here is not initialized by the
+		// GUI and therefore uses the default settings. This should not be a 
+		// problem but special snapping ranges configuration done in gui.cpp
+		// is ignored here... 
 				
 		m_old_invalidated_ranges.setNull();
 		add_invalidated_bounds(m_old_invalidated_ranges, true);
@@ -249,6 +212,13 @@ character::set_child_invalidated()
   } 
 }
 
+void 
+character::dump_character_tree(const std::string prefix) const
+{
+  log_debug("%s%s<%p> I=%d,CI=%d", prefix, typeName(*this).c_str(), this,
+    m_invalidated, m_child_invalidated);  
+}
+
 void
 character::extend_invalidated_bounds(const InvalidatedRanges& ranges)
 {
@@ -275,9 +245,9 @@ character::x_getset(const fn_call& fn)
 	}
 	else // setter
 	{
-		double newx = fn.arg(0).to_number(&(fn.env()));
+		double newx = fn.arg(0).to_number();
 		matrix m = ptr->get_matrix();
-		m.set_x_translation(infinite_to_fzero(PIXELS_TO_TWIPS(newx)));
+		m.set_x_translation(std::floor(std::infinite_to_fzero(PIXELS_TO_TWIPS(newx))));
 		ptr->set_matrix(m);
 		ptr->transformedByScript(); // m_accept_anim_moves = false; 
 	}
@@ -298,9 +268,9 @@ character::y_getset(const fn_call& fn)
 	}
 	else // setter
 	{
-		double newy = fn.arg(0).to_number(&(fn.env()));
+		double newy = fn.arg(0).to_number();
 		matrix m = ptr->get_matrix();
-		m.set_y_translation(infinite_to_fzero(PIXELS_TO_TWIPS(newy)));
+		m.set_y_translation(std::floor(std::infinite_to_fzero(PIXELS_TO_TWIPS(newy))));
 		ptr->set_matrix(m);
 		ptr->transformedByScript(); // m_accept_anim_moves = false; 
 	}
@@ -324,7 +294,7 @@ character::xscale_getset(const fn_call& fn)
 	{
 		matrix m = ptr->get_matrix();
 
-		double scale_percent = fn.arg(0).to_number(&(fn.env()));
+		double scale_percent = fn.arg(0).to_number();
 
 		// Handle bogus values
 		if (isnan(scale_percent))
@@ -360,7 +330,7 @@ character::yscale_getset(const fn_call& fn)
 	{
 		matrix m = ptr->get_matrix();
 
-		double scale_percent = fn.arg(0).to_number(&(fn.env()));
+		double scale_percent = fn.arg(0).to_number();
 
 		// Handle bogus values
 		if (isnan(scale_percent))
@@ -396,7 +366,7 @@ character::xmouse_get(const fn_call& fn)
 		
 	m.transform_by_inverse(&b, a);
 
-	return as_value(TWIPS_TO_PIXELS(b.m_x));
+	return as_value(TWIPS_TO_PIXELS(b.x));
 }
 
 as_value
@@ -415,7 +385,7 @@ character::ymouse_get(const fn_call& fn)
 		
 	m.transform_by_inverse(&b, a);
 
-	return as_value(TWIPS_TO_PIXELS(b.m_y));
+	return as_value(TWIPS_TO_PIXELS(b.y));
 }
 
 as_value
@@ -430,9 +400,21 @@ character::alpha_getset(const fn_call& fn)
 	}
 	else // setter
 	{
+		as_value& inval = fn.arg(0);
+		double input = inval.to_number();
+		if ( inval.is_undefined() || inval.is_null() || ! isfinite(input) )
+		{
+			IF_VERBOSE_ASCODING_ERRORS(
+			log_aserror(_("Ignored attempt to set %s.%s=%s"),
+				ptr->getTarget(),
+				_("_alpha"), // trying to reuse translations
+				fn.arg(0).to_debug_string());
+			);
+			return rv;
+		}
 		// Set alpha modulate, in percent.
 		cxform	cx = ptr->get_cxform();
-		cx.m_[3][0] = infinite_to_fzero(fn.arg(0).to_number()) / 100.f;
+		cx.m_[3][0] = input / 100.f; // std::infinite_to_fzero(input) / 100.f;
 		ptr->set_cxform(cx);
 		ptr->transformedByScript(); // m_accept_anim_moves = false; 
 	}
@@ -484,18 +466,25 @@ character::width_getset(const fn_call& fn)
 	{
 		if ( ! bounds.isFinite() )
 		{
-			log_unimpl(_("FIXME: can't set _width on character with null or world bounds"));
+			log_unimpl(_("FIXME: can't set _width on character %s (%s) with null or world bounds"),
+				ptr->getTarget(), typeName(*ptr));
 			return rv;
 		}
 
 		double oldwidth = bounds.width();
-		assert(oldwidth>0);
+		if ( oldwidth <= 0 )
+		{
+			log_unimpl(_("FIXME: can't set _width on character %s (%s) with width %d"),
+				ptr->getTarget(), typeName(*ptr), oldwidth);
+			return rv;
+		}
 
-		double newwidth = PIXELS_TO_TWIPS(fn.arg(0).to_number(&(fn.env())));
+		double newwidth = PIXELS_TO_TWIPS(fn.arg(0).to_number());
 		if ( newwidth <= 0 )
 		{
 			IF_VERBOSE_ASCODING_ERRORS(
-			log_aserror(_("Setting _width=%g of character %s (%s)"), newwidth/20, ptr->getTarget().c_str(), typeName(*ptr).c_str());
+			log_aserror(_("Setting _width=%g of character %s (%s)"),
+				newwidth/20, ptr->getTarget(), typeName(*ptr));
 			);
 		}
 
@@ -529,14 +518,20 @@ character::height_getset(const fn_call& fn)
 	{
 		if ( ! bounds.isFinite() )
 		{
-			log_unimpl(_("FIXME: can't set _height on character with null or world bounds"));
+			log_unimpl(_("FIXME: can't set _height on character %s (%s) with null or world bounds"),
+				ptr->getTarget(), typeName(*ptr));
 			return rv;
 		}
 
 		double oldheight = bounds.height();
-		assert(oldheight>0);
+		if ( oldheight <= 0 )
+		{
+			log_unimpl(_("FIXME: can't set _height on character %s (%s) with height %d"),
+				ptr->getTarget(), typeName(*ptr), oldheight);
+			return rv;
+		}
 
-		double newheight = PIXELS_TO_TWIPS(fn.arg(0).to_number(&(fn.env())));
+		double newheight = PIXELS_TO_TWIPS(fn.arg(0).to_number());
 		if ( newheight <= 0 )
 		{
 			IF_VERBOSE_ASCODING_ERRORS(
@@ -572,7 +567,7 @@ character::rotation_getset(const fn_call& fn)
 		matrix m = ptr->get_matrix();
 
 		// input is in degrees
-		float rotation = (float) fn.arg(0).to_number(&(fn.env())) * float(M_PI) / 180.f;
+		float rotation = (float) fn.arg(0).to_number() * float(M_PI) / 180.f;
 		m.set_rotation(rotation);
 
 		ptr->set_matrix(m);
@@ -603,6 +598,32 @@ character::target_getset(const fn_call& fn)
 	return as_value(ptr->getTargetPath());
 }
 
+as_value
+character::name_getset(const fn_call& fn)
+{
+	boost::intrusive_ptr<character> ptr = ensureType<character>(fn.this_ptr);
+
+	if ( fn.nargs == 0 ) // getter
+	{
+		VM& vm = VM::get(); // TODO: fetch VM from ptr 
+		const std::string& name = ptr->get_name();
+		if ( vm.getSWFVersion() < 6 && name.empty() )
+		{
+			return as_value();
+		} 
+		else
+		{
+			return as_value(name.c_str());
+		}
+	}
+	else // setter
+	{
+		ptr->set_name(fn.arg(0).to_string().c_str());
+	}
+
+	return as_value();
+}
+
 void
 character::set_event_handlers(const Events& copyfrom)
 {
@@ -625,7 +646,7 @@ character::add_event_handler(const event_id& id, const action_buffer& code)
 {
 	_event_handlers[id].push_back(&code);
 
-	//log_msg(_("Setting handler for event %s"), id.get_function_name().c_str());
+	//log_debug(_("Setting handler for event %s"), id.get_function_name().c_str());
 
 	// Set the character as a listener iff the
 	// kind of event is a KEY or MOUSE one 
@@ -639,7 +660,7 @@ character::add_event_handler(const event_id& id, const action_buffer& code)
 		case event_id::MOUSE_UP:
 		case event_id::MOUSE_DOWN:
 		case event_id::MOUSE_MOVE:
-			//log_msg(_("Registering character as having mouse events"));
+			//log_debug(_("Registering character as having mouse events"));
 			has_mouse_event();
 			break;
 		default:
@@ -668,65 +689,59 @@ character::get_event_handler(const event_id& id) const
 	return handler;
 }
 
-character::~character()
-{
-}
-
-void
+bool
 character::unload()
 {
+	if ( ! _unloaded )
+	{
+		queueEvent(event_id::UNLOAD, movie_root::apDOACTION);
+	}
+
+	bool hasEvent = hasEventHandler(event_id::UNLOAD);
+
 	_unloaded = true;
-	//log_msg(_("Queuing unload event for character %p"), this);
-	queueEventHandler(event_id::UNLOAD);
-	//on_event(event_id::UNLOAD);
+
+	return hasEvent;
 }
 
 void
-character::queueEventHandler(const event_id& id)
+character::queueEvent(const event_id& id, int lvl)
 {
-	bool called=false;
 
-	movie_root& root = VM::get().getRoot();
+	movie_root& root = _vm.getRoot();
+	std::auto_ptr<ExecutableCode> event(new QueuedEvent(boost::intrusive_ptr<character>(this), id));
+	root.pushAction(event, lvl);
+}
 
-	std::auto_ptr<ExecutableCode> code ( get_event_handler(id) );
-	if ( code.get() )
-	{
-		root.pushAction(code);
-		called=true;
-	}
+bool
+character::hasEventHandler(const event_id& id) const
+{
+	Events::const_iterator it = _event_handlers.find(id);
+	if ( it != _event_handlers.end() ) return true;
 
-	// This is likely wrong, we use it as a workaround
-	// to the fact that we don't distinguish between
-	// ActionScript and SWF defined events
-	// (for example: onClipLoad vs. onLoad)
-	//
-	//if (called) return;
-
-	// Check for member function.
-	boost::intrusive_ptr<as_function> method = getUserDefinedEventHandler(id.get_function_name());
-	if ( method )
-	{
-		root.pushAction(method, boost::intrusive_ptr<character>(this));
-	}
+	boost::intrusive_ptr<as_function> method = getUserDefinedEventHandler(id.get_function_key());
+	if (method) return true;
 
+	return false;
 }
 
 boost::intrusive_ptr<as_function>
 character::getUserDefinedEventHandler(const std::string& name) const
 {
-	std::string method_name = name;
-	if ( _vm.getSWFVersion() < 7 )
-	{
-		boost::to_lower(method_name, _vm.getLocale());
-	}
+	string_table::key key = _vm.getStringTable().find(PROPNAME(name));
+	return getUserDefinedEventHandler(key);
+}
 
+boost::intrusive_ptr<as_function>
+character::getUserDefinedEventHandler(string_table::key key) const 
+{
 	as_value tmp;
 
 	boost::intrusive_ptr<as_function> func;
 
 	// const cast is needed due to getter/setter members possibly
 	// modifying this object even when only get !
-	if ( const_cast<character*>(this)->get_member(method_name, &tmp) )
+	if ( const_cast<character*>(this)->get_member(key, &tmp) )
 	{
 		func = tmp.to_as_function();
 	}
@@ -769,6 +784,7 @@ character::computeTargetPath() const
 	Path path;
 
 	// Build parents stack
+	const character* topLevel = 0;
 	const character* ch = this;
 	for (;;)
 	{
@@ -779,6 +795,7 @@ character::computeTargetPath() const
 		{
 			// it is completely legal to set root's _name
 			//assert(ch->get_name().empty());
+			topLevel = ch;
 			break;
 		}
 
@@ -786,10 +803,24 @@ character::computeTargetPath() const
 		ch = parent;
 	} 
 
-	if ( path.empty() ) return "/";
+	assert(topLevel);
+
+	if ( path.empty() )
+	{
+		if ( _vm.getRoot().getRootMovie() == this ) return "/";
+		std::stringstream ss;
+		ss << "_level" << m_depth-character::staticDepthOffset;
+		return ss.str();
+	}
 
 	// Build the target string from the parents stack
 	std::string target;
+	if ( topLevel != _vm.getRoot().getRootMovie() )
+	{
+		std::stringstream ss;
+		ss << "_level" << topLevel->get_depth()-character::staticDepthOffset;
+		target = ss.str();
+	}
 	for ( Path::reverse_iterator
 			it=path.rbegin(), itEnd=path.rend();
 			it != itEnd;
@@ -835,7 +866,7 @@ character::getTarget() const
 		{
 			assert(dynamic_cast<const movie_instance*>(ch));
 			std::stringstream ss;
-			ss << "_level" << ch->get_depth();
+			ss << "_level" << ch->get_depth()-character::staticDepthOffset;
 			path.push_back(ss.str());
 			// it is completely legal to set root's _name
 			//assert(ch->get_name().empty());
@@ -862,12 +893,189 @@ character::getTarget() const
 	return target;
 }
 
+#if 0
 /*public*/
 std::string
 character::get_text_value() const
 {
 	return getTarget();
 }
+#endif
+
+void
+character::destroy()
+{
+	// in case we are destroyed w/out being unloaded first
+	// see bug #21842
+	_unloaded = true;
+
+	/// we may destory a character that's not unloaded.
+	///(we don't have chance to unload it in current model, see new_child_in_unload_test.c)
+	/// We don't destroy ourself twice, right ?
+	assert(!_destroyed);
+	_destroyed = true;
+}
+
+void
+character::markCharacterReachable() const
+{
+	if ( m_parent ) m_parent->setReachable();
+	if ( _mask )
+	{
+		// Stop being masked if the mask was unloaded
+		if ( _mask->isUnloaded() )
+		{
+			const_cast<character*>(this)->setMask(0);
+		}
+		else _mask->setReachable();
+	}
+	if ( _maskee )
+	{
+		// Stop masking if the masked character was unloaded
+		if ( _maskee->isUnloaded() )
+		{
+			const_cast<character*>(this)->setMaskee(0);
+		}
+		else _maskee->setReachable();
+	}
+	markAsObjectReachable();
+}
+
+void
+character::setMask(character* mask)
+{
+	if ( _mask != mask )
+	{
+		set_invalidated();
+	}
+
+	// Backup this before setMaskee has a chance to change it..
+	character* prevMaskee = _maskee;
+
+	// If we had a previous mask unregister with it
+	if ( _mask && _mask != mask )
+	{
+		// the mask will call setMask(NULL) 
+		// on any previously registered maskee
+		// so we make sure to set our _mask to 
+		// NULL before getting called again
+		_mask->setMaskee(NULL);
+	}
+
+	// if we had a maskee, notify it to stop using
+	// us as a mask
+	if ( prevMaskee )
+	{
+		prevMaskee->setMask(0); 
+	}
+
+	// TODO: should we reset any original clip depth
+	//       specified by PlaceObject tag ?
+	set_clip_depth(noClipDepthValue); // this will set _mask !!
+	_mask = mask;
+	_maskee = 0;
+
+	if ( _mask )
+	{
+		log_debug(" %s.setMask(%s): registering with new mask %s",
+			getTarget().c_str(),
+			mask ? mask->getTarget().c_str() : "null",
+			_mask->getTarget().c_str());
+		/// Register as as masked by the mask
+		_mask->setMaskee(this);
+	}
+}
+
+/*private*/
+void
+character::setMaskee(character* maskee)
+{
+	if ( _maskee == maskee )
+	{
+		return;
+	}
+	if ( _maskee )
+	{
+		// We don't want the maskee to call setMaskee(null)
+		// on us again
+		log_debug(" %s.setMaskee(%s) : previously masked char %s being set as non-masked",
+			getTarget().c_str(), maskee ? maskee->getTarget().c_str() : "null", _maskee->getTarget().c_str());
+		_maskee->_mask = NULL;
+	}
+
+	_maskee = maskee;
+
+	if ( maskee )
+	{
+		set_clip_depth(dynClipDepthValue);
+	}
+	else
+	{
+		// TODO: should we reset any original clip depth
+		//       specified by PlaceObject tag ?
+		set_clip_depth(noClipDepthValue);
+	}
+}
+
+
+bool 
+character::boundsInClippingArea() const 
+{
+  geometry::Range2d<float> mybounds = getBounds();
+  get_world_matrix().transform(mybounds);
+  
+  return gnash::render::bounds_in_clipping_area(mybounds);  
+}
+
+#ifdef USE_SWFTREE
+character::InfoTree::iterator
+character::getMovieInfo(InfoTree& tr, InfoTree::iterator it)
+{
+	const std::string yes = _("yes");
+	const std::string no = _("no");
+
+	it = tr.append_child(it, StringPair(getTarget(), typeName(*this)));
+
+	std::ostringstream os;
+	os << get_depth();
+	tr.append_child(it, StringPair(_("Depth"), os.str()));
+
+        /// Don't add if the character has no ratio value
+        if (get_ratio() >= 0)
+        {
+            os.str("");
+            os << get_ratio();
+	        tr.append_child(it, StringPair(_("Ratio"), os.str()));
+	    }	    
+
+        /// Don't add if it's not a real clipping depth
+        if (int cd = get_clip_depth() != noClipDepthValue )
+        {
+		os.str("");
+		if (cd == dynClipDepthValue) os << "Dynamic mask";
+		else os << cd;
+
+		tr.append_child(it, StringPair(_("Clipping depth"), os.str()));	    
+        }
+
+        os.str("");
+        os << get_width() << "x" << get_height();
+	tr.append_child(it, StringPair(_("Dimensions"), os.str()));	
+
+	tr.append_child(it, StringPair(_("Dynamic"), isDynamic() ? yes : no));	
+	tr.append_child(it, StringPair(_("Mask"), isMaskLayer() ? yes : no));	    
+	tr.append_child(it, StringPair(_("Destroyed"), isDestroyed() ? yes : no));
+	tr.append_child(it, StringPair(_("Unloaded"), isUnloaded() ? yes : no));
+
+	return it;
+}
+#endif
+
+const sprite_instance*
+character::getAsRoot() const
+{
+    return get_root();
+}
 
 
 } // namespace gnash
