@@ -15,16 +15,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: drag_state.h,v 1.8 2008/01/21 20:55:50 rsavoye Exp $ */
 
 
 #ifndef GNASH_DRAG_STATE_H
 #define GNASH_DRAG_STATE_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "rect.h" // for composition
 #include "smart_ptr.h" // we keep character being dragged by intrusive_ptr
 
