@@ -23,6 +23,7 @@
 #ifndef GNASH_FONT_H
 #define GNASH_FONT_H
 
+#include "smart_ptr.h" // GNASH_USE_GC
 
 #include "types.h"
 #include "resource.h" // for inheritance of font
@@ -41,7 +42,7 @@ class tu_file;
 
 namespace gnash {
 
-class movie;
+class movie_definition;
 class shape_character_def;
 class stream;
 
@@ -110,8 +111,18 @@ class font : public resource
 public:
 	font();
 
-	// Create a device-font only font, using the given name to find it
-	font(const std::string& name);
+	/// Create a device-font only font, using the given name to find it
+	//
+	/// @param name
+	///	Name of the font face to look for.
+	///
+	/// @param bold
+	///	Whether to use the bold variant of the font.
+	///
+	/// @param italic
+	///	Whether to use the italic variant of the font.
+	///
+	font(const std::string& name, bool bold=false, bool italic=false);
 
 	~font();
 
@@ -226,11 +237,22 @@ public:
 	///
 	float	get_kerning_adjustment(int last_code, int this_code) const;
 
+	/// Return height of the EM square used for glyphs definition
+	//
+	/// @param embedded
+	///	If true, return is based on the SWF tag the font
+	///	was read from, otherwise will query the FreeTypeGlyphsProvider
+	///
+	unsigned short int unitsPerEM(bool embedded) const;
+
 	float	get_leading() const { return m_leading; }
 	float	get_descent() const { return m_descent; }
 
 	bool	is_subpixel_font() const { return m_subpixel_font; }
 	void	set_subpixel_font(bool isit) { m_subpixel_font = isit; }
+
+	bool	isBold() const { return m_is_bold; }
+	bool	isItalic() const { return m_is_italic; }
 private:
 
 	/// Read the table that maps from glyph indices to character codes.
