@@ -17,7 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: generic_character.cpp,v 1.13 2008/02/11 12:55:22 udog Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -55,7 +54,7 @@ generic_character::pointInShape(float x, float y) const
   matrix wm = get_world_matrix();
   point lp(x, y);
   wm.transform_by_inverse(lp);
-  return m_def->point_test_local(lp.x, lp.y);
+  return m_def->point_test_local(lp.x, lp.y, wm);
 }
 
 
