@@ -15,7 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: movie_root.h,v 1.109 2008/02/20 14:46:28 strk Exp $ */
 
 /// \page events_handling Handling of user events
 ///
@@ -67,22 +66,35 @@
 #define GNASH_MOVIE_ROOT_H
 
 #ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
+#include "gnashconfig.h" //USE_SWFTREE
 #endif
 
-#include "tu_config.h"
-
+#include "smart_ptr.h" // GNASH_USE_GC
+#include "dsodefs.h" // DSOEXPORT
 #include "mouse_button_state.h" // for composition
 #include "drag_state.h" // for composition
 #include "movie_instance.h" // for inlines
-//#include "timers.h" // for composition
 #include "asobj/Key.h"
 #include "smart_ptr.h" // for memory management
 #include "URL.h" // for loadMovie
 
+#ifdef USE_SWFTREE
+# include "tree.hh"
+#endif
+
+// GNASH_PARANOIA_LEVEL:
+// 0 : (not unimplemented)
+// 1 : quick assertions
+// 2 : add testInvariant
+//
+#ifndef GNASH_PARANOIA_LEVEL
+# define GNASH_PARANOIA_LEVEL 1
+#endif
+
 #include <vector>
 #include <list>
 #include <set>
+#include <bitset>
 
 // Forward declarations
 namespace gnash {
@@ -224,44 +236,30 @@ public:
     ///
     void set_display_viewport(int x0, int y0, int w, int h);
 
-    /// Get current viewport width, in pixels
-    unsigned getWidth() const
-    {
-        return m_viewport_width;
-    }
-
-    /// Get current viewport height, in pixels
-    unsigned getHeight() const
-    {
-        return m_viewport_height;
-    }
-
-    /// Set whether rescaling is allowed or not.
-    //
-    /// When rescaling is not allowed the Stage listeners
-    /// will get notified on any resize attempt.
-    ///
-    void allowRescaling(bool v)
-    {
-        _allowRescale=v;
-    }
-
-    bool isRescalingAllowed()
-    {
-        return _allowRescale;
-    }
+    /// \brief
+    /// Return the notional width of the stage, value depending
+    /// on scaleMode
+    unsigned getStageWidth() const;
 
     /// \brief
-        /// The host app can use this to tell the movie when
-        /// user's mouse pointer has moved.
+    /// Return the notional height of the stage, actual value depending
+    /// on scaleMode
+    unsigned getStageHeight() const;
+
+    /// \brief
+    /// The host app can use this to tell the movie when
+    /// user's mouse pointer has moved.
     //
-    /// Coordinates are in pixels.
+    /// Coordinates are in Stage Coordinate Space (pseudo-pixels units).
     ///
     /// This function should return TRUE iff any action triggered
     /// by the event requires redraw, see \ref events_handling for
     /// more info.
     ///
-        bool notify_mouse_moved(int x, int y);
+    /// TODO: take twips (or float pixels), or we won't be able to
+    ///       support sub-pixel accuracy in collision detection.
+    ///
+    bool notify_mouse_moved(int x, int y);
 
     /// \brief
     /// The host app can use this to tell the movie when the
@@ -293,7 +291,7 @@ public:
     ///
     bool notify_key_event(key::code k, bool down);
 
-	/// \brief
+    /// \brief
     /// Use this to retrieve the last state of the mouse, as set via
     /// notify_mouse_state(). 
     //
@@ -355,15 +353,6 @@ public:
 	return getRootMovie()->get_current_frame();
     }
 
-    /// \brief
-    /// Return the size of a logical movie pixel as
-    /// displayed on-screen, with the current device
-    /// coordinates.
-    float   get_pixel_scale() const
-    {
-        return m_pixel_scale;
-    }
-
     void set_background_color(const rgba& color);
 
     void set_background_alpha(float alpha);
@@ -474,6 +463,8 @@ public:
     void setFocus(character* ch);
     
     DSOEXPORT void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
+    
+    void dump_character_tree() const;
 
     /// Return the topmost active entity under the pointer
     //
@@ -502,6 +493,66 @@ public:
 
     bool testInvariant() const;
 
+    /// enum for the values of Stage.displayState
+    enum DisplayState {
+		normal,
+		fullScreen
+	};
+
+    /// enum for the values of Stage.scaleMode
+    enum ScaleMode {
+        showAll,
+        noScale,
+        exactFit,
+        noBorder
+    };
+
+    /// enum for horizonal position of the Stage
+    enum StageHorizontalAlign {
+        STAGE_H_ALIGN_C,
+        STAGE_H_ALIGN_L,
+        STAGE_H_ALIGN_R,
+    };
+
+    /// enum for vertical position of the Stage
+    enum StageVerticalAlign {
+        STAGE_V_ALIGN_C,
+        STAGE_V_ALIGN_T,       
+        STAGE_V_ALIGN_B
+    };
+
+    enum AlignMode {
+        STAGE_ALIGN_L,
+        STAGE_ALIGN_T,
+        STAGE_ALIGN_R,
+        STAGE_ALIGN_B
+    };
+
+    /// Sets movie_root's horizontal and vertical alignment to one
+    /// of the three possible positions for each dimension.
+    void setStageAlignment(short s);
+
+    typedef std::pair<StageHorizontalAlign, StageVerticalAlign> StageAlign;
+
+    /// Returns the current alignment of the stage (left/right/centre, top/
+    /// bottom/centre) as a std::pair
+    StageAlign getStageAlignment() const;
+
+    /// Sets the Stage object's align mode.
+    void setStageScaleMode(ScaleMode sm);
+    
+    /// Returns the Stage object's align mode.
+    ScaleMode getStageScaleMode() const { return _scaleMode; }
+
+    // The string representation of the current align mode.
+    std::string getStageAlignMode() const;
+
+    /// Returns the Stage object's align mode.
+    DisplayState getStageDisplayState() const { return _displayState; }
+
+    // The string representation of the current align mode.
+    void setStageDisplayState(const DisplayState ds);
+
     /// Action priority levels
     enum ActionPriorityLevel {
 
@@ -561,7 +612,9 @@ public:
     void addLiveChar(boost::intrusive_ptr<character> ch)
     {
 	// Don't register the object in the list twice 
+#if GNASH_PARANOIA_LEVEL > 1
 	assert(std::find(_liveChars.begin(), _liveChars.end(), ch) == _liveChars.end());
+#endif
         _liveChars.push_front(ch);
     }
 
@@ -657,6 +710,20 @@ public:
         return _hostfd;
     }
 
+	/// A callback to the GUI for sending events and receiving
+	/// replies.
+	static gnash::interfaceEventCallback interfaceHandle;
+
+	static void registerEventCallback(interfaceEventCallback handler)
+	{
+	   	interfaceHandle = handler;
+	}
+
+#ifdef USE_SWFTREE
+    typedef std::pair<std::string, std::string> StringPair;
+    void getMovieInfo(tree<StringPair>& tr, tree<StringPair>::iterator it);
+#endif
+
 private:
 
     /// A load movie request
@@ -754,6 +821,9 @@ private:
     /// Erase unloaded characters from the given listeners list
     static void cleanupUnloadedListeners(CharacterList& ll);
 
+    /// Cleanup references to unloaded characters and run the garbage collector.
+    void cleanupAndCollect();
+
     /// Push a character listener to the front of given container, if not already present
     static void add_listener(CharacterList& ll, character* elem);
 
@@ -780,9 +850,9 @@ private:
     /// Width and height of viewport, in pixels
     int         m_viewport_width, m_viewport_height;
 
-    float       m_pixel_scale;
-
     rgba        m_background_color;
+    bool	m_background_color_set;
+
     float       m_timer;
     int         m_mouse_x, m_mouse_y, m_mouse_buttons;
     void *      m_userdata;
@@ -839,10 +909,6 @@ private:
     ///
     bool fire_mouse_event();
 
-    /// If set to false, no rescale should be performed
-    /// when changing viewport size
-    bool _allowRescale;
-
     /// \brief
     /// Return the topmost entity covering the given point
     /// and enabled to receive mouse events.
@@ -949,6 +1015,12 @@ private:
     /// -1 if none
     ///
     int _hostfd;
+    
+    std::bitset<4> _alignMode;
+    
+    ScaleMode _scaleMode;
+    
+    DisplayState _displayState;
 };
 
 
