@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,16 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-
 #ifndef GNASH_BITMAPMOVIEDEFINITION_H
 #define GNASH_BITMAPMOVIEDEFINITION_H
 
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "movie_definition.h" // for inheritance
 #include "rect.h" // for composition
-#include "BitmapMovieInstance.h" // for create_instance
+#include "BitmapMovieInstance.h" // for create_movie_instance
 #include "bitmap_character_def.h" // for destructor visibility by intrusive_ptr
 #include "DynamicShape.h" // for destructor visibility by intrusive_ptr
 
@@ -41,14 +38,13 @@ namespace gnash
 
 /// A "movie" definition for a bitmap file
 //
-/// The create_instance function will return a BitmapMovieInstance
+/// The create_movie_instance function will return a BitmapMovieInstance
 ///
 class BitmapMovieDefinition : public movie_definition
 {
 	int _version;
 	rect _framesize;
 	size_t _framecount;
-	std::vector<PlayList> _playlist;
 	float _framerate;
 	std::string _url;
 
@@ -64,6 +60,8 @@ class BitmapMovieDefinition : public movie_definition
 	///
 	shape_character_def* getShapeDef();
 
+	size_t _bytesTotal;
+
 protected:
 
 #ifdef GNASH_USE_GC
@@ -87,7 +85,7 @@ public:
 	///  - Framesize extracted from image 
 	///  - Single frame (unlabeled)
 	///  - 12 FPS
-	///  - 0 bytes (for get_bytes_loaded()/get_bytes_total())
+	///  - image->size() bytes (for get_bytes_loaded()/get_bytes_total())
 	///  - provided url
 	///
 	BitmapMovieDefinition(std::auto_ptr<image::rgb> image, const std::string& url);
@@ -126,41 +124,40 @@ public:
 		return _framesize;
 	}
 
+	/// Return number of bytes loaded
+	//
+	/// Since no progressive load is implemented yet
+	/// we'll always return total bytes here..
+	///
 	virtual size_t get_bytes_loaded() const {
-		return 0;
+		return get_bytes_total();
 	}
 
+	/// Return total number of bytes which composed this movie
+	//
+	/// We actually cheat, and return the image size here...
+	///
 	virtual size_t get_bytes_total() const {
-		return 0;
+		return _bytesTotal;
 	}
 	
-	/// Create a playable sprite instance from a def.
-	virtual sprite_instance* create_instance()
-	{
-		return create_movie_instance();
-	}
-
 	/// Create a playable movie_instance from this def.
-	virtual movie_instance* create_movie_instance()
+	virtual movie_instance* create_movie_instance(character* parent=0)
 	{
-		return new BitmapMovieInstance(this);
-	}
-
-	virtual const PlayList& get_playlist(size_t frame_number) const
-	{
-		assert ( frame_number < _playlist.size() );
-		return _playlist[frame_number];
+		return new BitmapMovieInstance(this, parent);
 	}
 
 	virtual const std::string& get_url() const {
 		return _url;
 	}
 
-	// Inheritance from movie_definition requires this
-	// TODO: provide a default implementation in movie_definition instead ?
+	// Inheritance from movie_definition requires this.
+	// we always return 1 so sprite_instance::stagePlacementCallback
+	// doesn't skip our handling (TODO: check if it's correct to
+	// skip handling of 0-frames movies..).
 	size_t  get_loading_frame() const 
 	{
-		return 0;
+		return 1;
 	}
 };
 
