@@ -17,7 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: morph2_character_def.cpp,v 1.21 2008/01/21 20:56:00 rsavoye Exp $ */
 
 // Based on the public domain morph2.cpp of:
 // Thatcher Ulrich <tu@tulrich.com>, Mike Shaver <shaver@off.net> 2003,
@@ -100,7 +99,7 @@ private:
 
 	morph2_character_def::morph2_character_def():
 		m_last_ratio(-1.0f)
-	{
+	{   /* Markus: No delete??? */
 		m_shape1 = new shape_character_def();
 		m_shape2 = new shape_character_def();
 	}
@@ -114,16 +113,15 @@ private:
 	{
 //		GNASH_REPORT_FUNCTION;
 
-		unsigned int i;
-		float ratio = (inst->get_ratio()==character::noRatioValue) ? 0 : inst->get_ratio()/65535.0;
-
+		unsigned int i = 0;
+		float ratio = inst->get_ratio() / 65535.0;
 		// bounds
 		rect	new_bound;
 		new_bound.set_lerp(m_shape1->get_bound(), m_shape2->get_bound(), ratio);
 		set_bound(new_bound);
 
 		// fill styles
-		for (i=0; i < m_fill_styles.size(); i++)
+		for (; i < m_fill_styles.size(); i++)
 		{
 			fill_style& fs = m_fill_styles[i];
 
@@ -137,10 +135,11 @@ private:
 		for (i=0; i < m_line_styles.size(); i++)
 		{
 			line_style& ls = m_line_styles[i];
+
 			const line_style& ls1 = m_shape1->get_line_styles()[i];
 			const line_style& ls2 = m_shape2->get_line_styles()[i];
-			ls.m_width = (boost::uint16_t)frnd(flerp(ls1.get_width(), ls2.get_width(), ratio));
-			ls.m_color.set_lerp(ls1.get_color(), ls2.get_color(), ratio);
+
+			ls.set_lerp(ls1, ls2, ratio);
 		}
 
 		// This is used for cases in which number
@@ -174,10 +173,10 @@ private:
 				const edge& e1 = j < p1.size() ? p1[j] : empty_edge;
 				const edge& e2 = k < p2.size() ? p2[k] : empty_edge;
 
-				e.cp.x = flerp(e1.cp.x, e2.cp.x, ratio);
-				e.cp.y = flerp(e1.cp.y, e2.cp.y, ratio);
-				e.ap.x = flerp(e1.ap.x, e2.ap.x, ratio);
-				e.ap.y = flerp(e1.ap.y, e2.ap.y, ratio);
+				e.cp.x = static_cast<int>(flerp(e1.cp.x, e2.cp.x, ratio));
+				e.cp.y = static_cast<int>(flerp(e1.cp.y, e2.cp.y, ratio));
+				e.ap.x = static_cast<int>(flerp(e1.ap.x, e2.ap.x, ratio));
+				e.ap.y = static_cast<int>(flerp(e1.ap.y, e2.ap.y, ratio));
 				k++;
 				if (p2.size() <= k)
 				{
