@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,10 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-
 /// \page movie SWF Movies
 ///
 /// SWF Movies definitions are created by reading an SWF stream.
@@ -27,7 +23,7 @@
 /// multiple instances.
 ///
 /// A Movie definition is defined by the gnash::movie_definition class.
-/// A Movie instance is defined by the gnash::movie_interface class.
+/// A Movie instance is defined by the gnash::movie_instance class.
 /// 
 /// A Movie instance exposes the ActionScript
 /// Object base interface (gnash::as_object),
@@ -35,19 +31,23 @@
 ///
 /// The implementation of SWF parsing for a Movie definition
 /// is found in gnash::movie_def_impl::read.
+///
 /// Note that movie_definition is also used as a base class
-/// to sprite_definition, which is a sub-movie defined in an SWF
+/// for gnash::sprite_definition, which is a sub-movie defined in an SWF
 /// file. This seems to be the only reason to have a
 /// movie_def_impl class, being the top-level definition of
 /// a movie (the one with a CharacterDictionary in it).
 ///
+/// Also note that gnash::movie_instance is a subclass of gnash::sprite_instance,
+/// being the instance of a gnash::sprite_definition.
+///
+///
 
 
 #ifndef GNASH_MOVIE_DEFINITION_H
 #define GNASH_MOVIE_DEFINITION_H
 
 #include "character_def.h" // for inheritance
-#include "container.h"
 #include "fontlib.h"
 #include "jpeg.h"
 #include "tu_file.h"
@@ -57,7 +57,8 @@
 
 #include <string>
 #include <memory> // for auto_ptr
-#include <map>
+#include <vector> // for PlayList typedef
+#include <set> 
 
 // Forward declarations
 namespace gnash {
@@ -65,6 +66,7 @@ namespace gnash {
 	class bitmap_info;
 	class movie_instance;
 	class sprite_instance;
+	class ControlTag;
 }
 
 namespace gnash
@@ -93,7 +95,7 @@ namespace gnash
 class movie_definition : public character_def
 {
 public:
-	typedef std::vector<execute_tag*> PlayList;
+	typedef std::vector<ControlTag*> PlayList;
 
 	virtual int	get_version() const = 0;
 	virtual float	get_width_pixels() const = 0;
@@ -114,20 +116,13 @@ public:
 	/// the returned value should likely match the file size.
 	///
 	virtual size_t get_bytes_total() const = 0;
-	
-	/// Create a playable sprite_instance from a def.
-	//
-	/// This calls add_ref() on the movie_interface internally.
-	/// Call drop_ref() on the movie_interface when you're done with it.
-	/// Or use boost::intrusive_ptr<T> from base/smart_ptr.h if you want.
-	///
-	virtual sprite_instance* create_instance() = 0;
 
 	/// Create a movie instance from a def.
 	//
 	/// Not all movie definitions allow creation of
 	/// movie_instance. In particular, sprite_definition
-	/// can only create sprite_instance (see create_instance)
+	/// can only create sprite_instance, so will return NULL
+	/// from this method.
 	///
 	/// The default implementation returns NULL.
 	///
@@ -136,25 +131,11 @@ public:
 	/// movie_def_impl is one such example, future examples
 	/// should include jpeg_movie_def and similar..
 	///
-	virtual movie_instance* create_movie_instance() {
+	virtual movie_instance* create_movie_instance(character* /*parent*/=0)
+	{
 		return NULL;
 	}
 	
-	virtual void	output_cached_data(tu_file* /*out*/, const cache_options& /*options*/) {}
-	virtual void	input_cached_data(tu_file* /*in*/) {}
-	
-	/// \brief
-	/// Causes this movie def to generate texture-mapped
-	/// versions of all the fonts it owns. 
-	//
-	/// This improves
-	/// speed and quality of text rendering.  The
-	/// texture-map data is serialized in the
-	/// output/input_cached_data() calls, so you can
-	/// preprocess this if you load cached data.
-	///
-	virtual void	generate_font_bitmaps() {}
-	
 	//
 	// (optional) API to support gnash::create_movie_no_recurse().
 	//
@@ -232,23 +213,19 @@ public:
 	/// @param frame_number
 	///	 Frame number, 0-based (ie: first frame is 0)
 	///
-	virtual const PlayList& get_playlist(size_t frame_number) const = 0;
-
-	/// Return the list of init actions for given frame number
-	//
+	/// @return NULL if no execute tags are defined for the given frame number
+	///	    (the default implementation) or a pointer to the vector of them (PlayList)
 	///
-	/// @param frame_number
-	///	 Frame number, 0-based (ie: first frame is 0)
-	///
-	/// @return NULL if the given frame has no init actions associated
-	///         (this is the default behaviour)
-	///
-	virtual const PlayList* get_init_actions(size_t /*frame_number*/) {
-		return NULL;
+	virtual const PlayList* getPlaylist(size_t /*frame_number*/) const
+	{
+		return 0;
 	}
 
 	/// Get the named exported resource, if we expose it.
 	//
+	/// @param symbol
+	///	The symbol name. Matching should be case-insensitive for all SWF versions.
+	///
 	/// @return NULL if the label doesn't correspond to an exported
 	///         resource. This is the default behaviour.
 	///
@@ -258,6 +235,22 @@ public:
 	}
 
 
+	typedef std::pair<int, std::string> ImportSpec;
+	typedef std::vector< ImportSpec > Imports;
+
+	/// Import resources 
+	//
+	/// @param source
+	///	Movie containing the resources being imported
+	///
+	/// @param imports
+	///	Resources to import, each with the id to use in our dictionary
+	///
+	virtual void importResources(boost::intrusive_ptr<movie_definition> /*source*/, Imports& /*imports*/)
+	{
+	}
+
+
 	/// \brief
 	/// Get a character from the dictionary.
 	//
@@ -331,7 +324,7 @@ public:
 		return NULL;
 	}
 
-	/// Add an execute_tag to this movie_definition's playlist
+	/// Add an ControlTag to this movie_definition's playlist
 	//
 	/// The default implementation is a no-op.
 	///
@@ -341,25 +334,17 @@ public:
 	/// 	to the movie_def_impl.
 	///
 	/// TODO: take an auto_ptr.
-	/// NOTE: the default implementation just let the execute_tag leak.
+	/// NOTE: the default implementation just let the ControlTag leak.
 	///
-	virtual void	add_execute_tag(execute_tag* /*c*/)
-	{
-	}
-
-	/// Add an init action to the frame currently being loaded
-	//
-	/// The default implementation is a no-op.
-	///
-	virtual void	add_init_action(execute_tag* /*c*/)
+	virtual void	addControlTag(ControlTag* /*c*/)
 	{
 	}
 
 	/// Labels the frame currently being loaded with the given name. 
 	//
 	/// A copy of the name string is made and kept in this object.
-    /// In case of multiple frames with the same name, the last added
-    /// will be the one referenced by that name.
+	/// In case of multiple frames with the same name, the last added
+	/// will be the one referenced by that name.
 	///
 	/// The default implementation is a no-op.
 	///
@@ -516,20 +501,6 @@ public:
 
 	// ...
 
-	// This interface should be nice not to have in movie_definition
-	// The default returns DO_LOAD_BITMAPS.
-	virtual create_bitmaps_flag	get_create_bitmaps() const
-	{
-		return DO_LOAD_BITMAPS;
-	}
-
-	// This interface should be nice not to have in movie_definition
-	// The default returns DO_LOAD_FONT_SHAPES.
-	virtual create_font_shapes_flag	get_create_font_shapes() const
-	{
-		return DO_LOAD_FONT_SHAPES;
-	}
-
 	/// \brief
 	/// Return the URL of the SWF stream this definition has been read
 	/// from.
@@ -565,55 +536,6 @@ public:
 	virtual void load_next_frame_chunk() 
 	{
 	}
-
-	/// Add a depth to the timeline depth set in current frame 
-	//
-	/// The default implementations is a no-op
-	///
-	/// It is overridden by timeline-equipped definitions:
-	///	- sprite_definition
-	///	- movie_def_impl
-	///
-	/// @param depth
-	///	Depth of an instance placed by PlaceObject* tag.
-	///	Assumed to be in the static zone (an assertion would fail otherwise).
-	///
-	/// @param ratio
-	/// Ratio of an instance defined by PlaceObject2 tag, ranges from 0 to 65535
-	virtual void addTimelineDepth(int /*depth*/, int /*ratio*/) {}
-
-	/// Remove a depth from the timeline depth set in current frame 
-	//
-	/// The default implementations is a no-op
-	///
-	/// It is overridden by timeline-equipped definitions:
-	///	- sprite_definition
-	///	- movie_def_impl
-	///
-	/// @param depth
-	///	Depth of an instance placed by PlaceObject* tag.
-	///	Assumed to be in the static zone (an assertion would fail otherwise).
-	///
-	virtual void removeTimelineDepth(int /*depth*/) {}
-
-	/// \brief
-	/// Get the set of timeline depths supposed to contain
-	/// an instance in the given frame (0-based)
-	//
-	/// The default implementations is a no-op
-	///
-	/// PRECONDITIONS:
-	///	- Target frame was completely parsed.
-	///
-	/// @param frameno
-	///	0-based frame number we want to inspect.
-	///	Assumed to be in the static zone.
-	///
-	/// @param depths
-	///	A vector to copy all frame depths to.
-	///
-	virtual void getTimelineDepths(size_t /*frameno*/, std::map<int, int>& /*depths*/) 
-	{}
     
 };
 
