@@ -20,7 +20,7 @@
 #define GNASH_STREAM_H
 
 #include "swf.h"
-#include "tu_config.h" // still neded ?
+#include "dsodefs.h" // still neded ?
 #include "GnashException.h"
 
 #include <string>
@@ -108,11 +108,17 @@ public:
 	///
 	float	read_short_sfixed();
 
-	/// Read a 16bit floating point value
+	/// Read a 16bit (1:sign 5:exp 10:mantissa) floating point value
 	//
 	/// aligned read
 	///
-	float	read_float();
+	float	read_short_float();
+
+	/// Read a 32bit (1:sign 8:exp 23:mantissa) floating point value
+	//
+	/// aligned read
+	///
+	float	read_long_float();
 
 	/// Read a 64-bit double value
 	//
@@ -358,19 +364,7 @@ public:
 	///
 	/// NOTE: if GNASH_TRUST_SWF_INPUT is defined this function is a no-op 
 	///
-	void ensureBytes(unsigned long needed)
-	{
-#ifndef GNASH_TRUST_SWF_INPUT
-		if ( _tagBoundsStack.empty() ) return; // not in a tag (should we check file length ?)
-		unsigned long int left = get_tag_end_position() - get_position();
-		if ( left < needed )
-		{
-			std::stringstream ss;
-			ss << "premature end of tag: need to read " << needed << " bytes, but only " << left << " left in this tag";
-			throw ParserException(ss.str());
-		}
-#endif
-	}
+	void ensureBytes(unsigned long needed);
 
 	/// \brief
 	/// Ensure the requested number of bits are available for a bitwise read
