@@ -17,7 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 // 
 
-// $Id: video_stream_instance.cpp,v 1.47 2008/02/11 12:55:22 udog Exp $
 
 #include "sprite_instance.h"
 #include "video_stream_instance.h"
@@ -64,45 +63,45 @@ static void attachVideoProperties(as_object& o)
 {
 	//int target_version = o.getVM().getSWFVersion();
 
-	boost::intrusive_ptr<builtin_function> gettersetter;
+	as_c_function_ptr gettersetter;
 
-	gettersetter = new builtin_function(&character::x_getset, NULL);
+	gettersetter = &character::x_getset;
 	o.init_property("_x", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::y_getset, NULL);
+	gettersetter = &character::y_getset;
 	o.init_property("_y", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::xscale_getset, NULL);
+	gettersetter = &character::xscale_getset;
 	o.init_property("_xscale", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::yscale_getset, NULL);
+	gettersetter = &character::yscale_getset;
 	o.init_property("_yscale", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::xmouse_get, NULL);
+	gettersetter = &character::xmouse_get;
 	o.init_readonly_property("_xmouse", *gettersetter);
 
-	gettersetter = new builtin_function(&character::ymouse_get, NULL);
+	gettersetter = &character::ymouse_get;
 	o.init_readonly_property("_ymouse", *gettersetter);
 
-	gettersetter = new builtin_function(&character::alpha_getset, NULL);
+	gettersetter = &character::alpha_getset;
 	o.init_property("_alpha", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::visible_getset, NULL);
+	gettersetter = &character::visible_getset;
 	o.init_property("_visible", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::width_getset, NULL);
+	gettersetter = &character::width_getset;
 	o.init_property("_width", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::height_getset, NULL);
+	gettersetter = &character::height_getset;
 	o.init_property("_height", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::rotation_getset, NULL);
+	gettersetter = &character::rotation_getset;
 	o.init_property("_rotation", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::parent_getset, NULL);
+	gettersetter = &character::parent_getset;
 	o.init_property("_parent", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::target_getset, NULL);
+	gettersetter = &character::target_getset;
 	o.init_property("_target", *gettersetter, *gettersetter);
 }
 
@@ -164,11 +163,13 @@ video_stream_instance::video_stream_instance(video_stream_definition* def,
 {
 	//log_debug("video_stream_instance %p ctor", (void*)this);
 
-	if ( m_def ) _embeddedStream = true;
+	if ( m_def )
+	{
+		_embeddedStream = true;
+		attachVideoProperties(*this);
+	}
 
 	set_prototype(getVideoInterface());
-			
-	attachVideoProperties(*this);
 }
 
 video_stream_instance::~video_stream_instance()
@@ -284,6 +285,18 @@ void video_class_init(as_object& global)
 	global.init_member("Video", cl.get());
 }
 
+geometry::Range2d<float>
+video_stream_instance::getBounds() const
+{
+	if (_embeddedStream) return m_def->get_bound().getRange();
+
+	geometry::Range2d<float> bounds; // null bounds..
+
+	// TODO: return the bounds of the dynamically
+	//       loaded video if not embedded ?
+	return bounds;
+}
+
 #ifdef GNASH_USE_GC
 void
 video_stream_instance::markReachableResources() const
