@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,30 +15,24 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 // Implementation and helpers for SWF actions.
 
 
 #ifndef GNASH_ACTION_H
 #define GNASH_ACTION_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "tu_config.h"
+#include "dsodefs.h" // for DSOEXPORT
 
 #include "gnash.h"
 #include "as_object.h"
 #include "types.h"
-#include "container.h"
 #include "smart_ptr.h"
 
 #include <cwchar>
 
+#ifdef __sgi
+	__SGI_LIBC_USING_FROM_STD(va_list) 
+#endif
 
 namespace gnash {
 	class sprite_instance;
@@ -85,7 +79,7 @@ namespace gnash {
 	///
 	DSOEXPORT as_value call_method(const as_value& method, as_environment* env,
 		as_object* this_ptr, // this is ourself
-		int nargs, int first_arg_bottom_index);
+		int nargs, int first_arg_bottom_index, as_object* super=NULL);
 
 	const char*	call_method_parsed(
 		as_environment* env,
@@ -101,48 +95,6 @@ namespace gnash {
 	// 
 	//void register_as_object(const char* object_name, as_c_function_ptr handler);
 
-	/// Numerical indices for standard member names.  Can use this
-	/// to help speed up get/set member calls, by using a switch()
-	/// instead of nasty string compares.
-	enum as_standard_member
-	{
-		M_INVALID_MEMBER = -1,
-		M_X,
-		M_Y,
-		M_XSCALE,
-		M_YSCALE,
-		M_CURRENTFRAME,
-		M_TOTALFRAMES,
-		M_ALPHA,
-		M_VISIBLE,
-		M_WIDTH,
-		M_HEIGHT,
-		M_ROTATION,
-		M_TARGET,
-		M_FRAMESLOADED,
-		M_NAME,
-		M_DROPTARGET,
-		M_URL,
-		M_HIGHQUALITY,
-		M_FOCUSRECT,
-		M_SOUNDBUFTIME,
-		M_XMOUSE,
-		M_YMOUSE,
-		M_PARENT,
-		M_TEXT,
-		M_TEXTWIDTH,
-		M_TEXTCOLOR,
-		M_ONLOAD,
-		M_ONROLLOVER,
-		M_ONROLLOUT,
-
-		AS_STANDARD_MEMBER_COUNT
-	};
-
-	/// Return the standard enum, if the arg names a standard member.
-	/// Returns M_INVALID_MEMBER if there's no match.
-	as_standard_member	get_standard_member(const std::string& name);
-
 	// deprecated, use sprite_instance::loadMovie
 	//void attach_extern_movie(const char* c_url, const sprite_instance* target, const sprite_instance* root_movie);
 
