@@ -38,16 +38,29 @@
 #define OUTPUT_VERSION 7
 #define OUTPUT_FILENAME "DefineEditTextTest.swf"
 
-SWFDisplayItem add_text_field(SWFMovie mo, SWFBlock font, const char* text);
+SWFDisplayItem add_text_field(SWFMovie mo, SWFBlock font, const char* text,
+	float indent, float leftMargin, float rightMargin, SWFTextFieldAlignment align,
+	float lineSpacing,
+	unsigned int textR, unsigned int textG, unsigned int textB, unsigned int textA);
 
 SWFDisplayItem
-add_text_field(SWFMovie mo, SWFBlock font, const char* text)
+add_text_field(SWFMovie mo, SWFBlock font, const char* text, float indent,
+		float leftMargin, float rightMargin,
+		SWFTextFieldAlignment align, float lineSpacing,
+		unsigned int textR, unsigned int textG,
+		unsigned int textB, unsigned int textA)
 {
   SWFTextField tf;
 
   tf = newSWFTextField();
 
   SWFTextField_setFont(tf, font);
+  SWFTextField_setIndentation(tf, indent);
+  SWFTextField_setLeftMargin(tf, leftMargin);
+  SWFTextField_setRightMargin(tf, rightMargin);
+  SWFTextField_setAlignment(tf, align);
+  SWFTextField_setLineSpacing(tf, lineSpacing);
+  SWFTextField_setColor(tf, textR, textG, textB, textA);
 
   /* setting flags seem unneeded */
   /*SWFTextField_setFlags(tf, SWFTEXTFIELD_USEFONT|SWFTEXTFIELD_NOEDIT);*/
@@ -103,7 +116,7 @@ main(int argc, char** argv)
     return 1;
   }
 
-  sprintf(fdbfont, "%s/Bitstream Vera Sans.fdb", srcdir);
+  sprintf(fdbfont, "%s/Bitstream-Vera-Sans.fdb", srcdir);
 
   puts("Setting things up");
 
@@ -143,15 +156,15 @@ main(int argc, char** argv)
     SWFBrowserFont bfont = newSWFBrowserFont("_sans");
     SWFFont efont = loadSWFFontFromFile(font_file);
 
-    it = add_text_field(mo, (SWFBlock)bfont, "Hello");
+    it = add_text_field(mo, (SWFBlock)bfont, "Hello", 1, 2, 3, SWFTEXTFIELD_ALIGN_LEFT, 10, 100, 101, 102, 50);
     SWFDisplayItem_setName(it, "dtext1");
     SWFDisplayItem_moveTo(it, 0, 200);
-    it = add_text_field(mo, (SWFBlock)efont, "Hello");
+    it = add_text_field(mo, (SWFBlock)efont, "Hello", 4, 5, 6, SWFTEXTFIELD_ALIGN_CENTER, 11, 110, 111, 112, 51);
     SWFDisplayItem_setName(it, "etext1");
     SWFDisplayItem_moveTo(it, 0, 300);
 
     SWFBrowserFont bfont2 = newSWFBrowserFont("times");
-    it = add_text_field(mo, (SWFBlock)bfont2, "Hello");
+    it = add_text_field(mo, (SWFBlock)bfont2, "Hello", 7, 8, 9, SWFTEXTFIELD_ALIGN_RIGHT, 12, 120, 121, 122, 52);
     SWFDisplayItem_setName(it, "dtext2");
     SWFDisplayItem_moveTo(it, 0, 400);
   }
@@ -161,11 +174,18 @@ main(int argc, char** argv)
   check_equals(mo, "etext1.embedFonts", "true");
 
   check_equals(mo, "dtext1.__proto__", "TextField.prototype");
+  check_equals(mo, "etext1.__proto__", "TextField.prototype");
+
   // checks after placing some swf defined TextField
   check(mo, "TextField.prototype.hasOwnProperty('background')");
   check(mo, "TextField.prototype.hasOwnProperty('backgroundColor')");
   xcheck(mo, "TextField.prototype.hasOwnProperty('text')");
   check(mo, "TextField.prototype.hasOwnProperty('textColor')");
+  xcheck(mo, "!TextField.prototype.hasOwnProperty('_parent')");
+  xcheck(mo, "!TextField.prototype.hasOwnProperty('_xmouse')");
+  xcheck(mo, "!TextField.prototype.hasOwnProperty('_ymouse')");
+  xcheck(mo, "!TextField.prototype.hasOwnProperty('_xscale')");
+  xcheck(mo, "!TextField.prototype.hasOwnProperty('_yscale')");
   
   check_equals(mo, "typeof(dtext1)", "'object'");
   check_equals(mo, "typeof(dtext1.text)", "'string'");
@@ -187,8 +207,18 @@ main(int argc, char** argv)
   check_equals(mo, "dtext1.hasOwnProperty('backgroundColor')", "false");
   check_equals(mo, "dtext1.hasOwnProperty('textColor')", "false");
   check_equals(mo, "dtext1.hasOwnProperty('_alpha')", "false");
+  check(mo, "!dtext1.hasOwnProperty('_parent')");
+  check(mo, "!dtext1.hasOwnProperty('_xmouse')");
+  check(mo, "!dtext1.hasOwnProperty('_ymouse')");
+  check(mo, "!dtext1.hasOwnProperty('_xscale')");
+  check(mo, "!dtext1.hasOwnProperty('_yscale')");
+  check(mo, "!etext1.hasOwnProperty('_parent')");
+  check(mo, "!etext1.hasOwnProperty('_xmouse')");
+  check(mo, "!etext1.hasOwnProperty('_ymouse')");
+  check(mo, "!etext1.hasOwnProperty('_xscale')");
+  check(mo, "!etext1.hasOwnProperty('_yscale')");
   
-  xcheck_equals(mo, "dtext1.__proto__.hasOwnProperty('text')", "true");
+  xcheck(mo, "dtext1.__proto__.hasOwnProperty('text')");
   check_equals(mo, "dtext1.__proto__.hasOwnProperty('background')", "true");
   check_equals(mo, "dtext1.__proto__.hasOwnProperty('backgroundColor')", "true");
   check_equals(mo, "dtext1.__proto__.hasOwnProperty('textColor')", "true");
@@ -204,13 +234,75 @@ main(int argc, char** argv)
   check_equals(mo, "dtext1.backgroundColor", "0xffffff");
   check_equals(mo, "etext1.backgroundColor", "0xffffff");
   check_equals(mo, "dtext2.backgroundColor", "0xffffff");
-  check_equals(mo, "dtext1.textColor", "0x000000");
-  check_equals(mo, "etext1.textColor", "0x000000");
-  check_equals(mo, "dtext2.textColor", "0x000000");
+  check_equals(mo, "dtext1.textColor", "6579558");
+  check_equals(mo, "etext1.textColor", "7237488");
+  check_equals(mo, "dtext2.textColor", "7895418");
   check_equals(mo, "dtext1._alpha", "100");
   check_equals(mo, "etext1._alpha", "100");
   check_equals(mo, "dtext2._alpha", "100");
-  
+  check_equals(mo, "etext1._parent", "_root");
+  check_equals(mo, "dtext2._parent", "_root");
+  check_equals(mo, "etext1._xscale", "100");
+  check_equals(mo, "dtext2._xscale", "100");
+  check_equals(mo, "etext1._yscale", "100");
+  check_equals(mo, "dtext2._yscale", "100");
+  check_equals(mo, "typeof(etext1._xmouse)", "'number'");
+  check_equals(mo, "typeof(dtext2._xmouse)", "'number'");
+  check_equals(mo, "typeof(etext1._ymouse)", "'number'"); 
+  check_equals(mo, "typeof(dtext2._ymouse)", "'number'"); 
+
+  // TextFormat objects are created on the fly
+  add_actions(mo,
+	"etext1.tf = etext1.getTextFormat();"
+	"dtext2.tf = dtext2.getTextFormat();"
+	);
+  check_equals(mo, "typeof(etext1.tf)", "'object'"); 
+  check_equals(mo, "typeof(dtext2.tf)", "'object'"); 
+  check_equals(mo, "etext1.tf.size", "12"); 
+  check_equals(mo, "dtext2.tf.size", "12"); 
+  check_equals(mo, "etext1.tf.font", "'Bitstream Vera Sans'");
+  check_equals(mo, "dtext2.tf.font", "'times'"); 
+  check_equals(mo, "typeof(etext1.tf.bold)", "'boolean'");
+  check_equals(mo, "typeof(dtext2.tf.bold)", "'boolean'");
+  check_equals(mo, "etext1.tf.bold", "false");
+  check_equals(mo, "dtext2.tf.bold", "false"); 
+  check_equals(mo, "typeof(etext1.tf.italic)", "'boolean'");
+  check_equals(mo, "typeof(dtext2.tf.italic)", "'boolean'");
+  check_equals(mo, "etext1.tf.italic", "false");
+  check_equals(mo, "dtext2.tf.italic", "false"); 
+  check_equals(mo, "typeof(etext1.tf.indent)", "'number'");
+  check_equals(mo, "typeof(dtext2.tf.indent)", "'number'");
+  check_equals(mo, "etext1.tf.indent", "4");
+  check_equals(mo, "dtext2.tf.indent", "7"); 
+  check_equals(mo, "typeof(etext1.tf.leftMargin)", "'number'");
+  check_equals(mo, "typeof(dtext2.tf.leftMargin)", "'number'");
+  check_equals(mo, "etext1.tf.leftMargin", "5");
+  check_equals(mo, "dtext2.tf.leftMargin", "8"); 
+  check_equals(mo, "typeof(etext1.tf.rightMargin)", "'number'");
+  check_equals(mo, "typeof(dtext2.tf.rightMargin)", "'number'");
+  check_equals(mo, "etext1.tf.rightMargin", "6");
+  check_equals(mo, "dtext2.tf.rightMargin", "9"); 
+  check_equals(mo, "typeof(etext1.tf.align)", "'string'");
+  check_equals(mo, "typeof(dtext2.tf.align)", "'string'");
+  check_equals(mo, "etext1.tf.align", "'center'");
+  check_equals(mo, "dtext2.tf.align", "'right'"); 
+  check_equals(mo, "typeof(etext1.tf.leading)", "'number'");
+  check_equals(mo, "typeof(dtext2.tf.leading)", "'number'");
+  check_equals(mo, "etext1.tf.leading", "11");
+  check_equals(mo, "dtext2.tf.leading", "12"); 
+  check_equals(mo, "typeof(etext1.tf.color)", "'number'");
+  check_equals(mo, "typeof(dtext2.tf.color)", "'number'");
+  check_equals(mo, "etext1.tf.color", "7237488");
+  check_equals(mo, "dtext2.tf.color", "7895418");
+  check_equals(mo, "typeof(etext1.tf.blockIndent)", "'number'");
+  check_equals(mo, "typeof(dtext2.tf.blockIndent)", "'number'");
+  check_equals(mo, "etext1.tf.blockIndent", "0");
+  check_equals(mo, "dtext2.tf.blockIndent", "0");
+  check_equals(mo, "typeof(etext1.tf.underline)", "'boolean'");
+  check_equals(mo, "typeof(dtext2.tf.underline)", "'boolean'");
+  check_equals(mo, "etext1.tf.underline", "false");
+  check_equals(mo, "dtext2.tf.underline", "false");
+
   add_actions(mo, "dtext1.background = true;"
                   "etext1.background = true;"
                   "dtext2.background = true;"
@@ -224,6 +316,9 @@ main(int argc, char** argv)
                   "etext1.text += ' world';"
                   "dtext2.text += ' world';" );
 
+  check_equals(mo, "etext1.getTextFormat().color", "0xff00ff");
+  check_equals(mo, "dtext2.getTextFormat().color", "0xffff00"); 
+
   check_equals(mo, "dtext1.text", "'Hello world'");
   check_equals(mo, "etext1.text", "'Hello world'");
   check_equals(mo, "dtext2.text", "'Hello world'");
