@@ -16,7 +16,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: processor.cpp,v 1.84.2.1 2008/03/03 16:53:25 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -25,14 +24,14 @@
 #include <iostream>
 #include <cstdio>
 #include <sys/time.h>
-#include <time.h>
+#include <ctime>
 
 #ifdef ENABLE_NLS
-#include <locale.h>
+#include <locale>
 #endif
 
 #include "gettext.h"
-#include "tu_file.h"
+#include "ClockTime.h"
 #include "gnash.h"
 #include "movie_definition.h"
 #include "sprite_instance.h"
@@ -45,10 +44,14 @@
 #include "VM.h"
 #include "noseek_fd_adapter.h"
 #include "ManualClock.h"
-#include "tu_timer.h"
+#include "StringPredicates.h"
 
 extern "C"{
 	#include <unistd.h>
+#ifdef _WIN32
+#include <windows.h>
+#define usleep(t) Sleep((t) / 1000)
+#endif
 #ifdef HAVE_GETOPT_H
 	#include <getopt.h>
 #endif
@@ -111,15 +114,15 @@ double lastAdvanceTimer;
 void
 resetLastAdvanceTimer()
 {
-	using namespace tu_timer;
-	lastAdvanceTimer = ticks_to_seconds(get_ticks());
+    // clocktime::getTicks() returns milliseconds
+	lastAdvanceTimer = static_cast<double>(clocktime::getTicks()) / 1000.0;
 }
 
 double
 secondsSinceLastAdvance()
 {
-	using namespace tu_timer;
-	double now = ticks_to_seconds(get_ticks());
+    // clocktime::getTicks() returns milliseconds
+	double now = static_cast<double>(clocktime::getTicks()) / 1000.0;
 	return ( now - lastAdvanceTimer);
 }
 
@@ -128,11 +131,60 @@ secondsSinceLastAdvance()
 //
 static int quitrequested = false;
 
-void execFsCommand(sprite_instance* movie, const char* command, const char* args)
+void execFsCommand(sprite_instance* movie, const std::string& command, const std::string& args)
 {
     log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
 
-    if ( ! strcasecmp(command, "quit") ) quitrequested=true;
+    StringNoCaseEqual ncasecomp;
+   
+    if ( ncasecomp(command, "quit") ) quitrequested = true;
+}
+
+std::string eventCallback(const std::string& event,
+							const std::string& arg)
+{
+    log_debug(_("eventCallback: %s %s"), event, arg);
+
+    static bool mouseShown = true;
+
+    // These should return "true" if the mouse was visible before
+    // the call.
+    if ( event == "Mouse.hide" ) {
+        bool state = mouseShown;
+        mouseShown = false;
+        return state ? "true" : "false";
+    }
+
+    if ( event == "Mouse.show" ) {
+        bool state = mouseShown;
+        mouseShown = true;
+        return state ? "true" : "false" ;
+    }
+    
+    // Some fake values for consistent test results.
+    
+    if ( event == "System.capabilities.screenResolutionX" ) {
+        return "800";
+    }
+
+    if ( event == "System.capabilities.screenResolutionY" ) {
+        return "640";
+    } 
+
+    if ( event == "System.capabilities.screenDPI" ) {
+        return "72";
+    }        
+
+    if ( event == "System.capabilities.screenColor" ) {
+        return "Color";
+    } 
+
+    if ( event == "System.capabilities.playerType" ) {
+        return "StandAlone";
+    } 
+
+    return "";
+
 }
 
 int
@@ -164,12 +216,18 @@ main(int argc, char *argv[])
  
     std::vector<const char*> infiles;
  
-    RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-    rcfile.loadFiles();
+    //RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+    //rcfile.loadFiles();
     
     if (rcfile.verbosityLevel() > 0) {
         dbglogfile.setVerbosity(rcfile.verbosityLevel());
     }
+
+    dbglogfile.setLogFilename(rcfile.getDebugLog());
+
+    if (rcfile.useWriteLog()) {
+        dbglogfile.setWriteDisk(true);
+    }
     
     if (rcfile.useActionDump()) {
         dbglogfile.setActionDump(true);
@@ -265,7 +323,8 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    register_fscommand_callback(execFsCommand);
+    registerFSCommandCallback(execFsCommand);
+    gnash::movie_root::registerEventCallback(&eventCallback);
 
     // Play through all the movies.
     for (int i = 0, n = infiles.size(); i < n; i++) {
@@ -347,7 +406,7 @@ play_movie(const char* filename)
     long clockAdvance = fpsDelay/1000;
     long localDelay = delay == -1 ? fpsDelay : delay; // microseconds
 
-    printf("Will sleep %ld microseconds between iterations - fps is %g, clockAdvance is %lu\n", localDelay, fps, clockAdvance);
+    log_debug("Will sleep %ld microseconds between iterations - fps is %g, clockAdvance is %lu\n", localDelay, fps, clockAdvance);
 
     // Use a clock advanced at every iteration to match exact FPS speed.
     ManualClock cl;
@@ -364,7 +423,7 @@ play_movie(const char* filename)
         return md;
     }
 
-    printf("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
+    log_debug("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
     usleep(localDelay);
     
     resetLastAdvanceTimer();
@@ -401,7 +460,7 @@ play_movie(const char* filename)
 	++nadvances;
 	if ( limit_advances && nadvances >= limit_advances)
 	{
-		printf("exiting after " SIZET_FMT " advances\n", nadvances);
+		log_debug("exiting after " SIZET_FMT " advances\n", nadvances);
 		break;
 	}
 
@@ -412,7 +471,7 @@ play_movie(const char* filename)
 	{
 		if ( allowed_end_hits && ++end_hitcount >= allowed_end_hits )
 		{
-			printf("exiting after " SIZET_FMT 
+			log_debug("exiting after " SIZET_FMT 
 			       " times last frame was reached\n", end_hitcount);
 	    		break;
 		}
@@ -453,7 +512,7 @@ play_movie(const char* filename)
 	    if ( last_frame > latest_frame ) latest_frame = last_frame;
 	    if ( ++loop_back_count > allowloopbacks )
 	    {
-		    printf(SIZET_FMT " loop backs; jumping one-after "
+		    log_debug(SIZET_FMT " loop backs; jumping one-after "
 				    "latest frame (" SIZET_FMT ")\n",
 				    loop_back_count, latest_frame+1);
 		    m.goto_frame(latest_frame + 1);
@@ -467,7 +526,7 @@ play_movie(const char* filename)
 	    resetLastAdvanceTimer();
 	}
 
-	printf("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
+	log_debug("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
 	usleep(localDelay);
     }
     
